/*
 * Copyright 2015 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.basis.exception;

import ru.ispras.retrascope.util.LogMessage;

/**
 * The unchecked exception to be thrown by Retrascope components to indicate an occurrence of a
 * programmatic error which is not addressed by the default {@code RuntimeException} subclasses.
 *
 * @author <a href="mailto:i.melnichenko@deltasolutions.ru">Igor Melnichenko</a>
 */
public class RetrascopeRuntimeException extends RuntimeException {
  private static final long serialVersionUID = -6160209876328290425L;

  /**
   * Constructs an exception with the specified message.
   *
   * @param message the message
   */
  public RetrascopeRuntimeException(final String message) {
    super(message);
  }

  /**
   * Constructs an exception with the specified message and root exception.
   *
   * @param message the message
   * @param cause the root exception
   */
  public RetrascopeRuntimeException(final String message, final Throwable cause) {
    super(message, cause);
  }

  /**
   * Constructs an exception that incapsulates the specified one.
   *
   * @param exception the exception
   */
  public RetrascopeRuntimeException(final Exception exception) {
    super(exception);
  }

  /**
   * Creates an exception object with message containing the specified prefix and an information
   * about the specified object.
   * @param prefix Exception message prefix.
   * @param object Object that's data are to be presented at the message.
   * @return Exception with message that is constructed from the specified parameters.
   */
  public static RetrascopeRuntimeException makeException(
      final Enum<?> prefix,
      final Object object) {
    return new RetrascopeRuntimeException(LogMessage.makeMessage(prefix, object));
  }
}
