/*
 * Copyright 2014 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.engine.basis;
 
import ru.ispras.retrascope.result.test.Vector;

import java.util.Iterator;

/**
 * The class representing an atomic test generator, that is, test generator operating at the
 * sequence-level instead of test-level.
 *
 * @author <a href="mailto:i.melnichenko@deltasolutions.ru">Igor Melnichenko</a>
 */
public abstract class AtomicTestGenerator {

  /**
   * Returns an iterator which enables to retrieve a next sequence of vectors produced by this
   * generator.
   * <br>
   * In subclasses which implement this method it is required to explicitly document the fact
   * whether the sequence is generated lazily (during iteration over the iterator) or not (upon a
   * call of the method).
   *
   * @return the iterator over the sequence
   */
  public abstract Iterator<Vector> getNextSequenceIterator();

  /**
   * Resets this generator. This method is used to start generation of a new test.
   */
  public abstract void reset();
}
