package ru.ispras.retrascope.engine.cfg.printer.graphml.depend;

import org.apache.commons.cli.ParseException;
import ru.ispras.retrascope.basis.Arguments;
import ru.ispras.retrascope.basis.Entity;
import ru.ispras.retrascope.basis.EntityType;
import ru.ispras.retrascope.basis.FileNames;
import ru.ispras.retrascope.basis.Parameter;
import ru.ispras.retrascope.engine.cfg.CfgEngine;
import ru.ispras.retrascope.model.cfg.CfgModel;
import ru.ispras.retrascope.model.cfg.walker.CfgWalkerFactory;
import ru.ispras.retrascope.model.cfg.walker.CfgWalkerMode;
import ru.ispras.retrascope.util.Log;

import java.util.Map;

/**
 * GraphML printer for dependencies in the Control Flow Graph model.
 * <p>The printer shows control dependencies between processes of
 * Control Flow Graph model.</p>
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public class CfgDependencyGraphMlPrinter extends CfgEngine {

  /**
   * Engine identifier.
   */
  public static final String ENGINE_ID = "cfg-dep-graphml-printer";

  /**
   * The engine output entity type.
   */
  public static final EntityType OUT_TYPE = EntityType.get("cfg-dep-graphml");

  /**
   * The command line parameters.
   */
  public static final Parameter OUT_FILE_NAME =
      new Parameter("cgaa-graphml", true, "Output GraphML file name");

  /* The default file name. */
  private static final String DEFAULT_FILE_NAME = "cfg-dependency-model.graphml";

  /* The printer result. */
  private FileNames fileNames;

  /**
   * Constructs object with the specified identifier, input/output types,
   * walkers.
   */
  public CfgDependencyGraphMlPrinter() {
    super(
        ENGINE_ID,
        CfgModel.TYPE,
        OUT_TYPE,
        CfgWalkerFactory.newWalker(new CfgDependencyGraphVisitor(), CfgWalkerMode.DFS_NO_RPT));
  }

  private FileNames getFileNames() {
    return this.fileNames;
  }

  private void setFileNames(final FileNames fileNames) {
    this.fileNames = fileNames;
  }

  private void initGraphMlVisitor(final String fileName) {
    ((CfgDependencyGraphVisitor)getWalkers()[0].getVisitor()).setFileName(fileName);
  }

  @Override
  public void initialize(Map<EntityType, Entity> inputs) {

    final Arguments arguments = (Arguments)inputs.get(Arguments.TYPE);

    try {

      /* Get arguments from command line. */
      final Arguments engineArgs = parseCommandLine(arguments.getCommandLine());

      /* Create GraphML file. */
      setFileNames(new FileNames("graphml"));
      fileNames.add(engineArgs.getValue(OUT_FILE_NAME, DEFAULT_FILE_NAME));

      initGraphMlVisitor(getFileNames().getList().get(0));

    } catch (ParseException e) {
      Log.printError(e.toString());
    }
  }

  @Override
  public Entity getOutput() {
    return getFileNames();
  }
}
