/*
 * Copyright 2014 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ru.ispras.retrascope.engine.cfg.transformer.cgaa;

import ru.ispras.retrascope.basis.Engine;
import ru.ispras.retrascope.basis.Entity;
import ru.ispras.retrascope.basis.EntityType;
import ru.ispras.retrascope.engine.cfg.CfgVisitorBackend;
import ru.ispras.retrascope.model.cfg.CfgModel;
import ru.ispras.retrascope.model.cfg.walker.CfgWalker;
import ru.ispras.retrascope.model.cfg.walker.CfgWalkerFactory;
import ru.ispras.retrascope.model.cfg.walker.CfgWalkerMode;
import ru.ispras.retrascope.model.cgaa.CgaaModel;

import java.util.Map;

/**
 * Transformer to clocked guarded atomic actions model.
 *
 * <p>The transformer of control flow graph model
 * to clocked guarded atomic actions in the form of
 * guarded actions decision diagram (GADD).</p>
 *
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public class CfgCgaaTransformer extends Engine {

  /**
   *  The engine identifier.
   */
  public static final String ENGINE_ID = "cfg-cgaa-transformer";

  /**
   * Default constructor.
   */
  public CfgCgaaTransformer() {
    super(CfgCgaaTransformer.ENGINE_ID);

    addInputType(CfgModel.TYPE);
    setOutputType(CgaaModel.TYPE);

    /* Engine back-ends registration. */
    addBackend(new CfgVisitorBackend(new CgaaAllConcurrentBackend(), CfgWalkerMode.DFS));
    addBackend(new CfgVisitorBackend(new CgaaFilterPathBackend(), CfgWalkerMode.DFS_NO_RPT));
  }

  @Override
  public Entity start(final Map<EntityType, Entity> inputs) {

    /* Walk the model with the specified visitor. */
    final CfgCgaaTransformerVisitor visitor = new CfgCgaaTransformerVisitor();
    final CfgWalker walker = CfgWalkerFactory.newWalker(visitor, CfgWalkerMode.DFS);
    walker.start((CfgModel) inputs.get(CfgModel.TYPE));

    /* Build a clocked guarded atomic actions model. */
    final CfgCgaaTransformerBuilder builder = new CfgCgaaTransformerBuilder(visitor.getModuleMap());
    return new CgaaModel(builder.build());
  }
}
