/*
 * Copyright 2014 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ru.ispras.retrascope.engine.cgaa.transformer.efsm;

import ru.ispras.fortress.expression.NodeVariable;
import ru.ispras.fortress.util.InvariantChecks;
import ru.ispras.retrascope.model.cfg.CfgDefaultVisitor;
import ru.ispras.retrascope.model.cfg.CfgProcess;

import java.util.Map;
import java.util.Set;

/**
 * Basic control flow graph visitor-based component.
 *
 * <p>Basic class for {@link ru.ispras.retrascope.model.cfg.CfgVisitor} visitors
 * that are used in {@link ru.ispras.retrascope.engine.cgaa.transformer.efsm.CgaaEfsmTransformer}.
 * All the successors of this class can use data containers (mappings) of different semantics
 * for transmitting data to each other.</p>
 *
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
class MapCfgVisitor extends CfgDefaultVisitor {

  /* The mapping of processes to their clock-like variables. */
  private Map<CfgProcess, Set<NodeVariable>> clockMap;

  /* The mapping of processes to their state-like variables. */
  private Map<CfgProcess, Set<NodeVariable>> stateMap;

  /**
   * Sets mapping of processes to their clock-like variables.
   *
   * @param map mapping of processes to their clock-like variables
   *
   * @throws IllegalArgumentException when argument is {@code null}.
   */
  public void setClockMap(final Map<CfgProcess, Set<NodeVariable>> map) {
    InvariantChecks.checkNotNull(map);

    this.clockMap = map;
  }

  /**
   * Returns the mapping of processes to their clock-like variables.
   *
   * @return mapping of processes to their clock-like variables
   */
  public Map<CfgProcess, Set<NodeVariable>> getClockMap() {
    return this.clockMap;
  }

  /**
   * Sets mapping of processes to their state-like variables.
   *
   * @param map mapping of processes to their state-like variables
   *
   * @throws IllegalArgumentException when argument is {@code null}.
   */
  public void setStateMap(final Map<CfgProcess, Set<NodeVariable>> map) {
    InvariantChecks.checkNotNull(map);

    this.stateMap = map;
  }

  /**
   * Returns the mapping of modules and their state-like variables.
   *
   * @return mapping of modules and their state-like variables
   */
  public Map<CfgProcess, Set<NodeVariable>> getStateMap() {
    return this.stateMap;
  }
}
