/*
 * Copyright 2014 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.engine.efsm.generator.test.fate;

import ru.ispras.retrascope.engine.efsm.generator.test.heuristic.direction.EfsmTransitionChooser;
import ru.ispras.retrascope.engine.efsm.generator.test.heuristic.direction.SemiRandomEfsmTransitionChooser;
import ru.ispras.retrascope.engine.efsm.simulator.ProcessSimulator;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.logging.Logger;

/**
 * The random FATE+ test generator.
 *
 * @author <a href="mailto:i.melnichenko@deltasolutions.ru">Igor Melnichenko</a>
 *
 * @see EfsmFateTestGenerator
 */
final class RandomFatePlusGenerator extends AbstractRandomFateGenerator {
  private long seed;

  /**
   * Constructs a FATE+ test generator for the specified EFSM.
   *
   * @param efsm the EFSM
   */
  RandomFatePlusGenerator(ProcessSimulator simulator) {
    super(simulator, "EFSM.FATE+.RandomGenerator." + simulator.getName());

    this.seed = 0;
  }

  @Override
  protected EfsmTransitionChooser getEfsmTransitionChooser() {
    this.seed++;

    return new SemiRandomEfsmTransitionChooser(super.simulator.getSimulatedModel(),
        super.simulator.getCurrentState(), super.simulator.getCoverage(), this.seed);
  }
}
