/*
 * Copyright 2015 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.engine.smv.test.parser;

import ru.ispras.retrascope.basis.Engine;
import ru.ispras.retrascope.basis.Entity;
import ru.ispras.retrascope.basis.EntityType;
import ru.ispras.retrascope.basis.FileNames;
import ru.ispras.retrascope.engine.smv.launcher.SmvModelCheckerLauncher;
import ru.ispras.retrascope.result.cfginterface.CfgInterface;
import ru.ispras.retrascope.result.test.Test;
import ru.ispras.retrascope.result.test.TestModel;

import java.util.Map;

/**
 * This engine parses the *.smvlog files generated by the model checker and constructs a new
 * {@link TestModel} entity.
 * 
 * @author <a href="mailto:lebedev@ispras.ru">Mikhail Lebedev</a>
 */
public final class SmvTestParser extends Engine {

  /** The engine ID. */
  public static final String ENGINE_ID = "smv-test-parser";

  /**
   * Constructs a new parser object.
   */
  public SmvTestParser() {
    super(ENGINE_ID);

    addInputType(SmvModelCheckerLauncher.OUT_TYPE);
    addInputType(CfgInterface.TYPE);
    setOutputType(TestModel.TYPE);
  }

  @Override
  public Entity start(Map<EntityType, Entity> inputs) {

    TestModel result = new TestModel();

    try {

      final FileNames files = (FileNames) inputs.get(SmvModelCheckerLauncher.OUT_TYPE);
      final CfgInterface iface = (CfgInterface) inputs.get(CfgInterface.TYPE);

      final SmvLogParser parser = new SmvLogParser();
      parser.setSignalInterface(iface);

      if (files != null) {
        for (String fileName : files.getList()) {
          final Test test = parser.parseCounterexamples(fileName);
          result.addTest(test);
        }
      }

    } catch (Exception e) {
      e.printStackTrace();
    }
    return result;
  }
  
}
