/*
 * Copyright 2015 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ru.ispras.retrascope.model.basis;

import ru.ispras.fortress.expression.printer.ExprTreePrinter;

/**
 * Object that can be characterized by it string identifier and printed
 * in some text format.
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public interface Identifiable {

  /**
   * Returns unique node identifier in string form.
   *
   * @return Node identifier.
   */
  String getId();

  /**
   * Returns a string description in the format of the specified printer.
   *
   * @param printer Printer that provides an output string format.
   *
   * @return String description.
   */
  String getDescription(ExprTreePrinter printer);

  /**
   * Returns a {@link String} description of the object in the format of the default system printer.
   *
   * @return String description.
   */
  String getDescription();
}
