/*
 * Copyright 2015 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.model.cfg;

import ru.ispras.retrascope.util.LogMessage;

import java.util.List;

/**
 * Statement that does not contain any other statements.
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
abstract class CfgAtomicStatement extends CfgSequentialStatement {

  protected CfgAtomicStatement() {
    super();
  }

  protected CfgAtomicStatement(CfgAtomicStatement statement) {
    super(statement);
  }

  @Override
  public int getStatementNum() {
    return 0;
  }

  @Override
  public CfgStatement getStatement(final int num) {
    return null;
  }

  @Override
  public void add(final CfgStatement statement) {
    throw new UnsupportedOperationException(
        LogMessage.makeMessage(LogMessage.NO_CHILD, this.getClass()));
  }

  @Override
  public void remove(final CfgStatement statement) {
    throw new UnsupportedOperationException(
        LogMessage.makeMessage(LogMessage.NO_CHILD, this.getClass()));
  }

  @Override
  public void addAll(final List<CfgStatement> list) {
    throw new UnsupportedOperationException(
        LogMessage.makeMessage(LogMessage.NO_CHILD, this.getClass()));
  }
}
