/*
 * Copyright 2015 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ru.ispras.retrascope.model.cfg;

import ru.ispras.fortress.util.InvariantChecks;
import ru.ispras.retrascope.basis.exception.RetrascopeRuntimeException;
import ru.ispras.retrascope.util.LogMessage;

import java.util.List;

/**
 * Class for control flow graph processes internal statements.
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
abstract class CfgSequentialStatement extends CfgStatement {

  protected CfgSequentialStatement() {
    super();
  }

  protected CfgSequentialStatement(CfgSequentialStatement statement) {
    super(statement);
  }

  @Override
  public void add(final CfgStatement statement) {

    if (!(statement instanceof CfgSequentialStatement)) {
      throw RetrascopeRuntimeException.makeException(LogMessage.WRONG_TYPE, statement);
    }
    super.add(statement);
  }

  @Override
  public void remove(final CfgStatement statement) {

    if (!(statement instanceof CfgSequentialStatement)) {
      throw RetrascopeRuntimeException.makeException(LogMessage.WRONG_TYPE, statement);
    }
    super.remove(statement);
  }

  @Override
  public void addAll(final List<CfgStatement> list) {
    InvariantChecks.checkNotNull(list);

    for (final CfgStatement statement : list) {
      add(statement);
    }
  }
}
