/*
 * Copyright 2015 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ru.ispras.retrascope.model.cfg;

import ru.ispras.fortress.data.DataType;
import ru.ispras.fortress.expression.Node;
import ru.ispras.fortress.expression.NodeVariable;
import ru.ispras.fortress.expression.printer.ExprTreePrinter;
import ru.ispras.retrascope.basis.exception.RetrascopeRuntimeException;
import ru.ispras.retrascope.model.basis.EventList;
import ru.ispras.retrascope.util.LogMessage;

import java.util.Collections;
import java.util.List;
import java.util.Set;

/**
 * Wait statement of the control flow graph model.
 *
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public class CfgWaitStatement extends CfgConditionStatement {

  public CfgWaitStatement(EventList list, Node condition) {
    super(list, condition);
    checkNode(condition);
  }

  public CfgWaitStatement(final EventList list) {
    super(list, null);
  }

  public CfgWaitStatement(final Node node) {
    super(node);
    checkNode(node);
  }

  private CfgWaitStatement(final CfgWaitStatement statement) {
    super(statement.getEventList(), statement.getCondition());
  }

  @Override
  public CfgStatementType getType() {
    return CfgStatementType.WAIT;
  }

  @Override
  public CfgStatement deepCopy() {
    return new CfgWaitStatement(this);
  }

  @Override
  public int getStatementNum() {
    return 0;
  }

  @Override
  public CfgStatement getStatement(final int num) {
    return null;
  }

  @Override
  public void add(final CfgStatement statement) {
    unsupported();
  }

  @Override
  public void remove(final CfgStatement statement) {
    unsupported();
  }

  @Override
  public void addAll(final List<CfgStatement> list) {
    unsupported();
  }

  private void unsupported() {
    throw new UnsupportedOperationException(
      LogMessage.makeMessage(LogMessage.NO_CHILD, this.getClass()));
  }

  @Override
  public String getDescription(final ExprTreePrinter printer) {
    return String.format(
        "%s %s",
        getType().name(),
        (getEventList().isEmpty()
            ? ""
            : getEventList()) + (getCondition() == null
                ? "" : " until " + printer.toString(getCondition())));
  }

  private static void checkNode(Node node) {
    if (!node.isType(DataType.BOOLEAN)) {
      throw new RetrascopeRuntimeException(
          String.format(
              "%s%s; only %s is allowed.",
              LogMessage.WRONG_TYPE,
              node.getDataType(),
              DataType.BOOLEAN));
    }
  }

  @Override
  public Set<NodeVariable> getDefines() {
    return Collections.emptySet();
  }
}
