/*
 * Copyright 2016 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ru.ispras.retrascope.model.cfg.walker;

import ru.ispras.fortress.util.InvariantChecks;
import ru.ispras.retrascope.basis.exception.RetrascopeRuntimeException;
import ru.ispras.retrascope.model.cfg.CfgVisitor;
import ru.ispras.retrascope.util.LogMessage;

/**
 * Factory for control flow graph model walkers.
 * @see CfgWalker
 * @see ru.ispras.retrascope.model.cfg.CfgModel
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public final class CfgWalkerFactory {

  private CfgWalkerFactory() {
    /* Empty. */
  }

  /**
   * Creates control flow graph model walker with the specified parameters.
   *
   * @param visitor Visitor object for control flow graph model.
   * @param mode Walking mode.
   * @return Walker object for control flow grpah model with the specified parameters.
   *
   * @throws IllegalArgumentException when any argument is {@code null}.
   * @throws RetrascopeRuntimeException when unsupported mode type is used.
   */
  public static CfgWalker newWalker(final CfgVisitor visitor, final CfgWalkerMode mode) {
    InvariantChecks.checkNotNull(visitor);
    InvariantChecks.checkNotNull(mode);

    switch (mode) {
      case DFS:
      case DFS_NO_RPT:
        return new CfgDfsWalker(visitor, mode);
      case BFS:
        return new CfgBfsWalker(visitor);
      default:
        throw RetrascopeRuntimeException.makeException(LogMessage.UNSUPPORTED_TYPE, mode);
    }
  }
}
