/*
 * Copyright 2015 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ru.ispras.retrascope.model.efsm;

import ru.ispras.fortress.expression.Node;
import ru.ispras.retrascope.model.basis.Assignment;
import ru.ispras.retrascope.model.basis.Event;

/**
 * Default visitor for EfsmModel.
 * <p>
 * Does nothing on every sub-component of EfsmModel.
 * </p>
 * @see EfsmModelVisitor
 * @see EfsmModel
 *
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public class EfsmModelDefaultVisitor implements EfsmModelVisitor {

  @Override
  public void onEfsmModelBegin(final EfsmModel efsmModel) {
    /* Empty. */
  }

  @Override
  public void onEfsmModelEnd(final EfsmModel efsmModel) {
    /* Empty. */
  }

  @Override
  public void onEfsmModuleBegin(final EfsmModule efsmModule) {
    /* Empty. */
  }

  @Override
  public void onEfsmModuleEnd(final EfsmModule efsmModule) {
    /* Empty. */
  }

  @Override
  public void onEfsmBegin(final Efsm efsm) {
    /* Empty. */
  }

  @Override
  public void onEfsmEnd(final Efsm efsm) {
    /* Empty. */
  }

  @Override
  public void onStateBegin(final EfsmState state) {
    /* Empty. */
  }

  @Override
  public void onStateEnd(final EfsmState state) {
    /* Empty. */
  }

  @Override
  public void onTransitionBegin(final EfsmTransition transition) {
    /* Empty. */
  }

  @Override
  public void onTransitionEnd(final EfsmTransition transition) {
    /* Empty. */
  }

  @Override
  public void onGuardedActionBegin(final GuardedAction guardedAction) {
    /* Empty. */
  }

  @Override
  public void onGuardedActionEnd(final GuardedAction guardedAction) {
    /* Empty. */
  }

  @Override
  public void onEvent(final Event event) {
    /* Empty. */
  }

  @Override
  public void onGuardBegin(final Guard guard) {
    /* Empty. */
  }

  @Override
  public void onGuardEnd(final Guard guard) {
    /* Empty. */
  }

  @Override
  public void onAssignmentActionBegin(final Action action) {
    /* Empty. */
  }

  @Override
  public void onAssignmentActionEnd(final Action action) {
    /* Empty. */
  }

  @Override
  public void onAssignmentBegin(final Assignment statement) {
    /* Empty. */
  }

  @Override
  public void onAssignmentEnd(final Assignment statement) {
    /* Empty. */
  }

  @Override
  public void onExpression(final Node expression) {
    /* Empty. */
  }
}
