/*
 * Copyright 2014 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ru.ispras.retrascope.model.efsm;

import ru.ispras.fortress.util.InvariantChecks;

/**
 * The walker for the {@link EfsmModule} class.
 *
 * @author <a href="mailto:i.melnichenko@deltasolutions.ru">Igor Melnichenko</a>
 */
public class EfsmModuleWalker extends EfsmWalker {
  private final EfsmModuleVisitor visitor;

  /**
   * Constructs a walker for the specified visitor.
   *
   * @param visitor the visitor
   */
  public EfsmModuleWalker(EfsmModuleVisitor visitor) {
    super(visitor);
    this.visitor = visitor;
  }

  /**
   * Performs walking across the specified EFSM container.
   *
   * @param efsmContainer the EFSM container to be walked across
   *
   * @throws IllegalArgumentException if {@code efsmContainer} is {@code null}
   */
  public void start(EfsmModule efsmContainer) {
    InvariantChecks.checkNotNull(efsmContainer);

    this.visitor.onEfsmModuleBegin(efsmContainer);

    for (Efsm efsm: efsmContainer.getNonInitialEfsms()) {
      this.start(efsm);
    } 

    for (EfsmModule nestedContainer: efsmContainer.getNestedModules()) {
      this.start(nestedContainer);
    }

    this.visitor.onEfsmModuleEnd(efsmContainer);
  }
}
