/*
 * Copyright 2015 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.model.hldd;

import ru.ispras.fortress.util.InvariantChecks;
import ru.ispras.retrascope.basis.Entity;
import ru.ispras.retrascope.basis.EntityType;

import java.util.ArrayList;
import java.util.Collection;


/**
 * This class represents a HlddModel which consists of HlddModules, HlddProcesses and Hldds.
 * 
 * @author <a href="mailto:lebedev@ispras.ru">Mikhail Lebedev</a>
 */
public final class HlddModel extends Entity {

  // Entity type.
  public static final EntityType TYPE = EntityType.get("hldd");

  // Stored modules.
  private Collection<HlddModule> modules;

  /**
   * Constructs a new model entity.
   */
  public HlddModel() {
    super(TYPE);
    modules = new ArrayList<HlddModule>();
  }

  /**
   * Constructs a new model entity using the specified collection of modules.
   * 
   * @param modules - the specified collection of {@link HlddModule}.
   */
  public HlddModel(final Collection<HlddModule> modules) {
    super(TYPE);
    this.modules = new ArrayList<HlddModule>();
    addAllModules(modules);
  }

  @Override
  public void add(final Entity entity) {
    if (entity instanceof HlddModel) {
      addAllModules(((HlddModel) entity).getModules());
    } else {
      throw new IllegalArgumentException();
    }
  }

  /**
   * Returns the containing modules.
   * 
   * @return The collection of {@link HlddModule}.
   */
  public Collection<HlddModule> getModules() {
    return modules;
  }

  /**
   * Adds the specified {@link HlddModule} to the model.
   * 
   * @param module - the specified module.
   */
  public void addModule(final HlddModule module) {
    InvariantChecks.checkNotNull(module);
    modules.add(module);
  }

  /**
   * Adds the specified {@link HlddModule} collection to the model.
   * 
   * @param modules - the specified collection.
   */
  public void addAllModules(final Collection<HlddModule> modules) {
    for (HlddModule module : modules) {
      addModule(module);
    }
  }

}
