/*
 * Copyright 2015 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.model.hldd;


/**
 * This class represents the {@link HlddModel} visitor interface.
 * 
 * @author <a href="mailto:lebedev@ispras.ru">Mikhail Lebedev</a>
 */
public interface HlddVisitor {

  /**
   * Starts visiting a model.
   * 
   * @param model - the specified model.
   */
  void onModelBegin(HlddModel model);

  /**
   * Finishes visiting a model.
   * 
   * @param model - the specified model.
   */
  void onModelEnd(HlddModel model);

  /**
   * Starts visiting {@link HlddModule}.
   * 
   * @param module - the specified module.
   */
  void onModuleBegin(HlddModule module);

  /**
   * Finishes visiting {@link HlddModule}.
   * 
   * @param module - the specified module.
   */
  void onModuleEnd(HlddModule module);

  /**
   * Starts visiting {@link HlddProcess}.
   * 
   * @param hldds - the specified process.
   */
  void onProcessBegin(HlddProcess hldds);

  /**
   * Finishes visiting {@link HlddProcess}.
   *  
   * @param hldds - the specified process.
   */
  void onProcessEnd(HlddProcess hldds);

  /**
   * Starts visiting {@link Hldd}.
   * 
   * @param hldd - the specified hldd.
   */
  void onHlddBegin(Hldd hldd);

  /**
   * Finishes visiting {@link Hldd}.
   * 
   * @param hldd - the specified hldd.
   */
  void onHlddEnd(Hldd hldd);

  /**
   * Starts visiting the root node of a {@link Hldd}.
   * 
   * @param rootNode - the specified {@link HlddNode}.
   */
  void onRootBegin(HlddNode rootNode);

  /**
   * Finishes visiting the root node of a {@link Hldd}.
   * 
   * @param rootNode - the specified {@link HlddNode}.
   */
  void onRootEnd(HlddNode rootNode);

  /**
   * Starts visiting a node.
   * 
   * @param node - the specified {@link HlddNode}.
   */
  void onNodeBegin(HlddNode node);

  /**
   * Finishes visiting a node.
   * 
   * @param node - the specified {@link HlddNode}.
   */
  void onNodeEnd(HlddNode node);

  /**
   * Starts visiting a condition node.
   * 
   * @param node - the specified {@link HlddNode}.
   */
  void onConditionBegin(HlddNode node);

  /**
   * Finishes visiting a condition node.
   * 
   * @param node - the specified {@link HlddNode}.
   */
  void onConditionEnd(HlddNode node);

  /**
   * Starts visiting a value node.
   * 
   * @param node - the specified {@link HlddNode}.
   */
  void onValueBegin(HlddNode node);

  /**
   * Finishes visiting a value node.
   * 
   * @param node - the specified {@link HlddNode}.
   */
  void onValueEnd(HlddNode node);

  /**
   * Starts visiting an assignment node.
   * 
   * @param node - the specified {@link HlddNode}.
   */
  void onAssignmentBegin(HlddNode node);

  /**
   * Finishes visiting an assignment node.
   * 
   * @param node - the specified {@link HlddNode}.
   */
  void onAssignmentEnd(HlddNode node);

}
