/*
 * Copyright 2014 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ru.ispras.retrascope.parser.basis;

import ru.ispras.retrascope.basis.Engine;
import ru.ispras.retrascope.basis.Entity;
import ru.ispras.retrascope.basis.EntityType;
import ru.ispras.retrascope.engine.cfg.CfgVisitorBackend;
import ru.ispras.retrascope.model.cfg.CfgVisitor;
import ru.ispras.retrascope.model.cfg.walker.CfgWalkerMode;
import ru.ispras.retrascope.parser.basis.backend.CfgAssertBackend;
import ru.ispras.retrascope.parser.basis.backend.CfgBasicBlockSequenceBackend;
import ru.ispras.retrascope.parser.basis.backend.CfgLoopBackend;
import ru.ispras.retrascope.parser.basis.backend.CfgOneAssignmentProcessBackend;
import ru.ispras.retrascope.parser.basis.backend.CfgSwitchSequenceBackend;
import ru.ispras.retrascope.parser.basis.backend.CfgWaitBackend;

import java.util.Map;

/**
 * HDL parser basic class.
 *
 * <p>Returns {@link ru.ispras.retrascope.model.cfg.CfgModel}
 * control flow graph model representation.</p>
 *
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public abstract class HdlParser extends Engine {

  /**
   * Constructs parser with the specified identifier.
   *
   * @param parserId Parser identifier.
   */
  protected HdlParser(final String parserId) {
    super(parserId);

    /* Parser back-ends registration. */
    addNoRptBackend(new CfgLoopBackend());
    addNoRptBackend(new CfgAssertBackend());
    addNoRptBackend(new CfgWaitBackend());
    addNoRptBackend(new CfgBasicBlockSequenceBackend());
    addNoRptBackend(new CfgOneAssignmentProcessBackend());
    addNoRptBackend(new CfgSwitchSequenceBackend());
  }

  private void addNoRptBackend(final CfgVisitor visitor) {
    addVisitorBackend(visitor, CfgWalkerMode.DFS_NO_RPT);
  }

  private void addVisitorBackend(
      final CfgVisitor visitor,
      final CfgWalkerMode mode) {
    addBackend(new CfgVisitorBackend(visitor, mode));
  }

  /**
   * Parses input entities.
   *
   * @param inputs Input entities collection.
   *
   * @return Output entity.
   */
  public abstract Entity parse(final Map<EntityType, Entity> inputs);

  @Override
  public Entity start(final Map<EntityType, Entity> inputs) {
    return parse(inputs);
  }
}
