/*
 * Copyright 2015 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ru.ispras.retrascope.result.assertion;

/**
 * Assertion entity visitor.
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public interface AssertionVisitor {

  /**
   * Notifies that visiting process has been started.
   */
  void onRootBegin();

  /**
   * Notifies that visiting process has been finished.
   */
  void onRootEnd();

  /**
   * Starts visiting of the specified entity object.
   *
   * @param entity Entity to be visited.
   */
  void onEntityBegin(AssertionEntity entity);

  /**
   * Finishes visiting of the specified entity object.
   *
   * @param entity Entity that has been visited.
   */
  void onEntityEnd(AssertionEntity entity);

  /**
   * Starts visiting of the specified container object.
   *
   * @param container Container to be visited.
   */
  void onContainerBegin(AssertionContainer container);

  /**
   * Finishes visiting of the specified container object.
   *
   * @param container Container that has been visited.
   */
  void onContainerEnd(AssertionContainer container);

  /**
   * Notifies that group object has been visited.
   *
   * @param group Group that has been visited.
   */
  void onAssertionGroup(AssertionGroup group);
}
