/*
 * Copyright 2014 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.result.testbench;

import ru.ispras.retrascope.basis.Entity;
import ru.ispras.retrascope.basis.EntityType;
import ru.ispras.retrascope.basis.FileNames;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Set;

/**
 * The representation of a VHDL testbench.
 *
 * @author <a href="mailto:i.melnichenko@deltasolutions.ru">Igor Melnichenko</a>
 */
public class VhdlTestbenchModel extends FileNames {

  /**
   * The entity type ID of all {@code VhdlTestbench} instances.
   *
   * @see ru.ispras.retrascope.basis.EntityType#get(String)
   * @see Entity#Entity(ru.ispras.retrascope.basis.EntityType)
   */
  public static final EntityType TYPE = EntityType.get("vhdl-testbench");

  private final List<Set<Testbench>> testbenches;

  /**
   * Constructs object with the specified testbenches.
   *
   * @param testbenches testbenches
   */
  public VhdlTestbenchModel(final List<Set<Testbench>> testbenches) {
    super(VhdlTestbenchModel.TYPE.getId());

    for (final Set<Testbench> testbenchSet: testbenches) {
      for (final Testbench testbench: testbenchSet) {
        super.add(testbench.getTestbenchFile().toString());
      }
    }

    this.testbenches = new ArrayList<>(testbenches);
  }

  /**
   * Returns model testbenches.
   *
   * @return testbenches
   */
  public List<Set<Testbench>> getTestbenches() {
    return Collections.unmodifiableList(this.testbenches);
  }
}
