/*
 * Copyright 2014 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ru.ispras.retrascope;

import java.io.File;
import java.util.LinkedHashMap;
import java.util.Map;

/**
 * HDL test files for project.
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public class HdlTestFiles {

  private static final Map<String, Map<String,String>> VHDL_MAP = new LinkedHashMap<>();
  private static final Map<String, Map<String,String>> VERILOG_MAP = new LinkedHashMap<>();

  /* Directories names. */
  private static final String ITC99 = "itc99";

  private HdlTestFiles() {
    /* Empty. */
  }

  static {

    /* === VHDL test projects === */

    /* sample */
    final Map<String, String> bvSample = new LinkedHashMap<>();
    putToMap(bvSample, getFile("/bvsample/bvsample.vhd"));
    VHDL_MAP.put("bvsample", bvSample);

    /* sample */
    final Map<String, String> sample = new LinkedHashMap<>();
    putToMap(sample, getFile("/sample/sample.vhd"));
    VHDL_MAP.put("sample", sample);

    //TODO: uncomment 'count' project when functions processing will be implemented
    /* count */
//    Map<String, String> count = new LinkedHashMap<>();
//    putToMap(count, getFile("/count/count.vhd"));
//    VHDL_MAP.put("count", count);

    /* example */
    Map<String, String> example = new LinkedHashMap<>();
    putToMap(example, getFile("/example/test.vhd"));
    putToMap(example, getFile("/example/driver.vhd"));
    putToMap(example, getFile("/example/example.vhd"));
    VHDL_MAP.put("example", example);

    /* ITC'99  benchmark */
    final Map<String, String> itc99 = new LinkedHashMap<>();

    putToMap(itc99, getFile("/" + ITC99 + "/b01/b01.vhd"));
    putToMap(itc99, getFile("/" + ITC99 + "/b02/b02.vhd"));
    putToMap(itc99, getFile("/" + ITC99 + "/b03/b03.vhd"));
    putToMap(itc99, getFile("/" + ITC99 + "/b04/b04.vhd"));
//    putToMap(itc99, getFile("/" + ITC99 + "/b05/b05.vhd")); // more than 0.5 hour for CFG-CGAA
    putToMap(itc99, getFile("/" + ITC99 + "/b06/b06.vhd"));
    putToMap(itc99, getFile("/" + ITC99 + "/b07/b07.vhd"));
    putToMap(itc99, getFile("/" + ITC99 + "/b08/b08.vhd"));
    putToMap(itc99, getFile("/" + ITC99 + "/b09/b09.vhd"));
    putToMap(itc99, getFile("/" + ITC99 + "/b10/b10.vhd"));
    putToMap(itc99, getFile("/" + ITC99 + "/b11/b11.vhd"));
//    putToMap(itc99, getFile("/" + ITC99 + "/b12/b12.vhd"));
//    putToMap(itc99, getFile("/" + ITC99 + "/b13/b13.vhd"));
//    putToMap(itc99, getFile("/" + ITC99 + "/b14/b14.vhd"));
//    putToMap(itc99, getFile("/" + ITC99 + "/b14_1/b14_1.vhd"));
//    putToMap(itc99, getFile("/" + ITC99 + "/b15/b15.vhd"));
//    putToMap(itc99, getFile("/" + ITC99 + "/b15_1/b15_1.vhd"));
//    putToMap(itc99, getFile("/" + ITC99 + "/b17/b17.vhd"));
//    putToMap(itc99, getFile("/" + ITC99 + "/b17_1/b17_1.vhd"));
//    putToMap(itc99, getFile("/" + ITC99 + "/b18/b18.vhd"));
//    putToMap(itc99, getFile("/" + ITC99 + "/b18_1/b18_1.vhd"));
//    putToMap(itc99, getFile("/" + ITC99 + "/b19/b19.vhd"));
//    putToMap(itc99, getFile("/" + ITC99 + "/b19_1/b19_1.vhd"));
//    putToMap(itc99, getFile("/" + ITC99 + "/b20/b20.vhd"));
//    putToMap(itc99, getFile("/" + ITC99 + "/b20_1/b20_1.vhd"));
//    putToMap(itc99, getFile("/" + ITC99 + "/b21/b21.vhd"));
//    putToMap(itc99, getFile("/" + ITC99 + "/b21_1/b21_1.vhd"));
//    putToMap(itc99, getFile("/" + ITC99 + "/b22/b22.vhd"));
//    putToMap(itc99, getFile("/" + ITC99 + "/b22_1/b22_1.vhd"));
    VHDL_MAP.put("itc99", itc99);

    /* MiniMIPS */
    /*Map<String, String> minimips = new LinkedHashMap<>();
    putToMap(minimips, getFile("/minimips/alu.vhd"));
    putToMap(minimips, getFile("/minimips/banc.vhd"));
    putToMap(minimips, getFile("/minimips/bus_ctrl.vhd"));
    putToMap(minimips, getFile("/minimips/minimips.vhd"));
    putToMap(minimips, getFile("/minimips/pps_di.vhd"));
    putToMap(minimips, getFile("/minimips/pps_ei.vhd"));
    putToMap(minimips, getFile("/minimips/pps_ex.vhd"));
    putToMap(minimips, getFile("/minimips/pps_mem.vhd"));
    putToMap(minimips, getFile("/minimips/pps_pf.vhd"));
    putToMap(minimips, getFile("/minimips/ram.vhd"));
    putToMap(minimips, getFile("/minimips/renvoi.vhd"));
    putToMap(minimips, getFile("/minimips/rom.vhd"));
    putToMap(minimips, getFile("/minimips/syscop.vhd"));
    VHDL_MAP.put("minimips", minimips);*/

    /* Plasma */
//    Map<String, String> plasma = new LinkedHashMap<>();
//    putToMap(plasma, getFile("/plasma/alu.vhd"));
//    putToMap(plasma, getFile("/plasma/bus_mux.vhd"));
//    putToMap(plasma, getFile("/plasma/control.vhd"));
//    putToMap(plasma, getFile("/plasma/mem_ctrl.vhd"));
//    putToMap(plasma, getFile("/plasma/mlite2sram.vhd"));
//    putToMap(plasma, getFile("/plasma/mlite2uart.vhd")); // records
//    putToMap(plasma, getFile("/plasma/mlite_cpu.vhd")); // bv_increment
//    putToMap(plasma, getFile("/plasma/mult.vhd")); // bv_inc6
//    putToMap(plasma, getFile("/plasma/pc_next.vhd")); // bv_increment
//    putToMap(plasma, getFile("/plasma/pipeline.vhd"));
//    putToMap(plasma, getFile("/plasma/plasma.vhd")); // records
//    putToMap(plasma, getFile("/plasma/ram.vhd")); // Zamia error
//    putToMap(plasma, getFile("/plasma/reg_bank.vhd")); // conv_integer
//    putToMap(plasma, getFile("/plasma/shifter.vhd"));
//    putToMap(plasma, getFile("/plasma/sram2mlite.vhd"));
//    putToMap(plasma, getFile("/plasma/uart.vhd")); // records
//    VHDL_MAP.put("plasma", plasma);

    /* === Verilog test projects === */

    /* adder */
    final Map<String, String> adder = new LinkedHashMap<>();
    putToMap(adder, getFile("/adder/adder4.v"));
//    putToMap(adder, getFile("/adder/adder4_testbench.v"));
    VERILOG_MAP.put("adder", adder);

    /* empty */
    final Map<String, String> empty = new LinkedHashMap<>();
    putToMap(empty, getFile("/empty/empty.v"));
    VERILOG_MAP.put("empty", empty);

    /* fifo */
    final Map<String, String> fifo = new LinkedHashMap<>();
    putToMap(fifo, getFile("/fifo/fifo.v"));
    putToMap(fifo, getFile("/fifo/fifo_testbench.v"));
    VERILOG_MAP.put("fifo", fifo);

    /* fifo_mod */
//    final Map<String, String> fifoMod = new LinkedHashMap<>();
//    putToMap(fifoMod, getFile("/fifo_mod/fifo.v"));
//    VERILOG_MAP.put("fifo_mod", fifoMod);

    /* for_loop */
    final Map<String, String> forLoop = new LinkedHashMap<>();
    putToMap(forLoop, getFile("/for_loop/for_loop.v"));
    VERILOG_MAP.put("for_loop", forLoop);

    /* incompat_cond */
    final Map<String, String> incompat = new LinkedHashMap<>();
    putToMap(incompat, getFile("/incompat_cond/incompat_cond.v"));
    VERILOG_MAP.put("incompat_cond", incompat);

    /* fifo0 */
//    Map<String, String> fifo0 = new LinkedHashMap<>();
//    putToMap(fifo0, getFile("/fifo0/mem_2p.v"));
//    putToMap(fifo0, getFile("/fifo0/mul_fifo.v"));
//    putToMap(fifo0, getFile("/fifo0/myfifo.v"));
//    VERILOG_MAP.put("fifo", fifo0);

    /* mux */
    final Map<String, String> mux = new LinkedHashMap<>();
    putToMap(mux, getFile("/mux/mux4_1.v"));
//    putToMap(mux, getFile("/mux/mux4_1_testbench.v"));
    VERILOG_MAP.put("mux", mux);

    /* ram */
    final Map<String, String> ram = new LinkedHashMap<>();
    putToMap(ram, getFile("/ram/ram.v"));
    putToMap(ram, getFile("/ram/ram_testbench.v"));
    VERILOG_MAP.put("ram", ram);
  }

  /**
   * Returns a mapping between projects and included VHDL file paths & top-level names.
   *
   * @return VHDL projects from the test suite.
   */
  public static Map<String, Map<String, String>> getVhdlProjects() {
    return VHDL_MAP;
  }

  /**
   * Returns a mapping between projects and included Verilog file paths & top-level names.
   *
   * @return Verilog projects from the test suite.
   */
  public static Map<String, Map<String, String>> getVerilogProjects() {
    return VERILOG_MAP;
  }

  /**
   * Returns a mapping of projects to their files.
   *
   * @return Projects from the test suite.
   */
  public static Map<String, Map<String, String>> getProjects() {
    final Map<String, Map<String, String>> map = new LinkedHashMap<>();
    map.putAll(VHDL_MAP);
    map.putAll(VERILOG_MAP);
    return map;
  }

  private static void putToMap(
      final Map<String, String> map,
      final String filePath) {

    final String ext = getExt(filePath);
    if (ext.equalsIgnoreCase("vhd")) {
      map.put(filePath, getNameWithoutExt(filePath));
    } else {
      map.put(filePath, "");
    }
  }

  private static String getExt(final String filePath) {
    return filePath.substring(filePath.lastIndexOf('.') + 1, filePath.length());
  }

  private static String getNameWithoutExt(final String fullName) {
    return fullName.substring(
      fullName.lastIndexOf(File.separatorChar) + 1, fullName.lastIndexOf('.'));
  }

  private static String getFile(final String resourceName) {
    return new File(HdlTestFiles.class.getResource(resourceName).getFile()).getAbsolutePath();
  }
}
