/*
 * Copyright 2016 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope;

import org.junit.Test;

import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.List;

/**
 * This class represents the base class for test cases which use the SMV model checker.
 * 
 * @author <a href="mailto:lebedev@ispras.ru">Mikhail Lebedev</a>
 */
public abstract class SmvUtilTest {

  private static List<String> excluded;

  static {
    excluded = new ArrayList<>();
//    excluded.add(Paths.get("b03", "b03.vhd").toString()); // nuSMV works too long
//    excluded.add(Paths.get("b04", "b04.vhd").toString()); // nuSMV works too long
    excluded.add(Paths.get("b05", "b05.vhd").toString()); // nuSMV works too long
//    excluded.add(Paths.get("b07", "b07.vhd").toString()); // array access out of bounds
    excluded.add(Paths.get("b08", "b08.vhd").toString()); // array access out of bounds
//    excluded.add(Paths.get("b10", "b10.vhd").toString()); // nuSMV works too long
//    excluded.add(Paths.get("b11", "b11.vhd").toString()); // nuSMV works too long
    excluded.add(Paths.get("b12", "b12.vhd").toString()); // array store operations
    excluded.add(Paths.get("b13", "b13.vhd").toString()); // hangs
//    excluded.add(Paths.get("empty", "empty.v").toString()); // senseless design
//    excluded.add(Paths.get("fifo", "fifo.v").toString());
//    excluded.add(Paths.get("fifo", "fifo_testbench.v").toString());
//    excluded.add(Paths.get("ram", "ram_testbench.v").toString());
    excluded.add(Paths.get("fifo_mod", "fifo.v").toString()); // assignment type mismatch
//    excluded.add(Paths.get("adder", "adder4.v").toString());
  }

  public SmvUtilTest() {  }

  @Test
  public void test() {
    runTest();
  }

  protected abstract void runTest();
  
  /**
   * Indicates whether the specified design path is not supported by the checker.
   * @param path - the specified design path (in the form {@code "folder/file"}).
   * @return {@code True} if the design is NOT supported, {@code false} otherwise.
   */
  public boolean isExcluded(final String path) {
    return excluded.contains(path);
  }

}
