/*
 * Copyright 2014 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ru.ispras.retrascope.engine.cfg.printer.graphml;

import org.junit.Test;
import ru.ispras.fortress.expression.printer.ExprPrinter;
import ru.ispras.retrascope.HdlTestFiles;
import ru.ispras.retrascope.HdlTestUtils;
import ru.ispras.retrascope.basis.Configuration;

import java.io.File;
import java.util.LinkedHashMap;
import java.util.Map;

/**
 * Module-level test for printer from control flow graph to GraphML format.
 *
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public class CfgGraphMlPrinterModuleTestCase {

  @Test
  public void runTest() {

    /* Get VHDL projects. */
    final Map<String, Map<String, String>> vhdlFiles = HdlTestFiles.getVhdlProjects();

    /* Get Verilog projects. */
    final Map<String, Map<String, String>> verilogFiles = HdlTestFiles.getVerilogProjects();

    /* Printing in all the defined styles. */

    final ExprPrinter[] printers = ExprPrinter.values();

    for (ExprPrinter printer : printers) {

      /* Run tool on each module of each VHDL project. */
      runToolOnFiles(vhdlFiles, printer);

      /* run tool on each module of each Verilog project. */
      runToolOnFiles(verilogFiles, printer);
    }
  }

  private void runToolOnFiles(
      final Map<String, Map<String, String>> hdlFiles,
      final ExprPrinter printer) {

    for (final Map.Entry<String, Map<String, String>> projectEntry : hdlFiles.entrySet()) {

      /* Get project container. */
      final Map<String, String> projectMap = projectEntry.getValue();

      for (final Map.Entry<String, String> entry : projectMap.entrySet()) {

        /* Only-one-file-containing mapping. */
        final Map<String, String> onlyFile = new LinkedHashMap<>();
        onlyFile.put(entry.getKey(), entry.getValue());

        /* Use design name for output GraphML file. */
        final String graphMlFileName = getFileName(entry.getKey());

        /* Save the result on the project directory. */
        HdlTestUtils.runVhdl(
            onlyFile,
            CfgGraphMlPrinter.OUT_TYPE.getId(),
            "--" + Configuration.PRINT_STYLE_PARAM.getLongName(),
            printer.name(),
            "--" + CfgGraphMlPrinter.OUT_FILE_NAME.getLongName(),
            graphMlFileName + "-" + printer.name() + "-cfg.graphml");
      }
    }
  }

  private String getFileName(final String fullPathName) {
    return fullPathName.substring(
        fullPathName.lastIndexOf(File.separatorChar) + 1,
        fullPathName.lastIndexOf('.'));
  }
}
