/*
 * Copyright 2015 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.engine.hldd.printer.smv;

import org.junit.Assert;

import ru.ispras.retrascope.HdlTestFiles;
import ru.ispras.retrascope.HdlTestUtils;
import ru.ispras.retrascope.SmvUtilTest;
import ru.ispras.retrascope.engine.hldd.printer.smv.HlddSmvPrinter;
import ru.ispras.retrascope.util.Log;
import ru.ispras.retrascope.util.LogMessage;
//import ru.ispras.retrascope.util.SmvChecker;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.PrintWriter;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Map.Entry;

/**
 * This class represents the test case for {@link HlddSmvPrinter} class.
 * 
 * @author <a href="mailto:lebedev@ispras.ru">Mikhail Lebedev</a>
 */
public final class HlddSmvPrinterTestCase extends SmvUtilTest {

  @Override
  protected void runTest() {

    final Map<String, Map<String, String>> files = HdlTestFiles.getProjects();

    for (Entry<String, Map<String, String>> project : files.entrySet()) {
      for (Entry<String, String> file : project.getValue().entrySet()) {

        final String filePath = file.getKey();

//        final String folderName =
//            filePath.substring(filePath.lastIndexOf(File.separator,
//                filePath.lastIndexOf(File.separator) - 1) + 1);

        final Map<String, String> onlyFile = new LinkedHashMap<>();
        onlyFile.put(filePath, file.getValue());
        final String designName =
            filePath.substring(filePath.lastIndexOf(File.separator) + 1, filePath.lastIndexOf('.'));
        final String smvFileName = designName + ".smv";

        /* If SMV file exists, clear it. */
        final File smvFile = new File(smvFileName);
        if (smvFile.exists()) {
          try {
            final PrintWriter writer = new PrintWriter(smvFile);
            writer.write("");
            writer.close();
          } catch (FileNotFoundException e) {
            Log.printError(LogMessage.makeMessage(LogMessage.CANT_FIND, smvFileName));
          }
        }

        HdlTestUtils.runVhdl(onlyFile, HlddSmvPrinter.OUT_TYPE.getId(), "--"
            + HlddSmvPrinter.SMV_FILE_NAME.getLongName(), smvFileName);
        
        if (!smvFileName.equals("empty.smv")) {
          Assert.assertTrue(exists(smvFileName));
        }

//        if (!isExcluded(folderName) && exists(smvFileName)) {
//          final SmvChecker checker = SmvChecker.getChecker();
//          checker.setNuXmv();
//          checker.runChecker(smvFileName, "-bmc"/*, "-iwls95", "1000"*/);
//          Assert.assertEquals(SmvChecker.Result.OK, checker.getResult());
//        }
      }
    }
  }

  private boolean exists(final String name) {
    final File file = new File(name);
    return file.exists();
  }

}
