/*
 * Copyright 2015 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.engine.smv;

import ru.ispras.retrascope.HdlTestFiles;
import ru.ispras.retrascope.HdlTestUtils;
import ru.ispras.retrascope.SmvUtilTest;
import ru.ispras.retrascope.engine.hldd.printer.smv.HlddSmvPrinter;
import ru.ispras.retrascope.engine.smv.launcher.SmvModelCheckerLauncher;

import java.io.File;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Map.Entry;

/**
 * This class represents the test case for {@link SmvModelCheckerLauncher} engine.
 * 
 * @author <a href="mailto:lebedev@ispras.ru">Mikhail Lebedev</a>
 */
public class SmvModelCheckerLauncherTestCase extends SmvUtilTest {

  @Override
  protected void runTest() {

    final Map<String, Map<String, String>> files = HdlTestFiles.getProjects();

    for (Entry<String, Map<String, String>> project : files.entrySet()) {
      for (Entry<String, String> file : project.getValue().entrySet()) {
        final String filePath = file.getKey();
        final Map<String, String> onlyFile = new LinkedHashMap<>();
        onlyFile.put(filePath, file.getValue());
        final String designName =
            filePath.substring(filePath.lastIndexOf(File.separator) + 1, filePath.lastIndexOf('.'));
        final String smvFileName = designName + ".smv";
        final String folderName =
            filePath.substring(filePath.lastIndexOf(File.separator,
                filePath.lastIndexOf(File.separator) - 1) + 1);
        if (!isExcluded(folderName)) {
          HdlTestUtils.runVhdl(onlyFile, SmvModelCheckerLauncher.OUT_TYPE.getId(), "--"
              + HlddSmvPrinter.SMV_FILE_NAME.getLongName(), smvFileName);
        }
      }
    }
  }

}
