/*
 * Copyright 2015 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.engine.test.printer.testbench.vhdl;

import org.junit.Test;

import ru.ispras.retrascope.HdlTestUtils;
import ru.ispras.retrascope.Retrascope;
import ru.ispras.retrascope.engine.efsm.generator.test.EfsmTestGenerator;
import ru.ispras.retrascope.result.testbench.VhdlTestbenchModel;

import java.io.IOException;
import java.nio.file.Paths;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;

/**
 * The test for VHDL testbench printer.
 *
 * @author <a href="mailto:i.melnichenko@deltasolutions.ru">Igor Melnichenko</a>
 *
 * @see TestVhdlTestbenchPrinter
 */
public class TestVhdlTestbenchPrinterVhdlTestCase {

  private static final String HYP = "--";

  /*
   * Designs which cause failures at the moment.
   */
  private static final String[] excludedDesigns = {
    Paths.get("bvsample", "bvsample.vhd").toString(),
    Paths.get("example", "test.vhd").toString(),
    Paths.get("example", "driver.vhd").toString(),
    Paths.get("example", "example.vhd").toString(),
    Paths.get("b05", "b05.vhd").toString(),
    Paths.get("b12", "b12.vhd").toString(),
    Paths.get("b13", "b13.vhd").toString(),
  };

  private static final Set<String> ignored = new HashSet<>(Arrays.asList(excludedDesigns));

  @Test
  public void generate() throws IOException {
    HdlTestUtils.runVhdl(
        VhdlTestbenchModel.TYPE.toString(),
        ignored,
        new String[] {HYP + Retrascope.ENGINE_PARAM.getLongName(), EfsmTestGenerator.ENGINE_ID,
            HYP + TestVhdlTestbenchPrinter.OVERWRITE_EXISTING_FILES_PARAMETER.getLongName(),
            HYP + EfsmTestGenerator.LOOP_LIMIT_PARAMETER.getLongName(), "25"//25 is for b11
        /*, "--" + Retrascope.LOG_LEVEL.getLongName(), "DEBUG"*/});
  }
}
