/*
 * Copyright 2015 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ru.ispras.retrascope.model.cfg;

import org.junit.Assert;
import org.junit.Test;
import ru.ispras.fortress.data.DataType;
import ru.ispras.fortress.expression.NodeValue;
import ru.ispras.fortress.expression.NodeVariable;
import ru.ispras.retrascope.util.SolverUtils;

/**
 * The test for {@link CfgUtils} class.
 *
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public class CfgUtilsTestCase {

  @Test
  public void runTest() {

    runIsTypeTest();
    runGetVarContainerTest();
  }

  private void runIsTypeTest() {

    CfgCaseStatement[] sameTypeNodes = new CfgCaseStatement[3];
    sameTypeNodes[0] = new CfgCaseStatement(NodeValue.newInteger(0));
    sameTypeNodes[1] = new CfgCaseStatement(NodeValue.newInteger(1));
    sameTypeNodes[2] = new CfgCaseStatement(NodeValue.newInteger(2));

    if (!CfgUtils.isType(CfgStatementType.CASE, sameTypeNodes)) {
      Assert.fail();
    }

    if (CfgUtils.isType(
        CfgStatementType.CASE,
        new CfgCaseStatement(SolverUtils.TRUE),
        new CfgSwitchStatement(SolverUtils.TRUE))) {
      Assert.fail();
    }
  }

  private void runGetVarContainerTest() {

    final CfgModule root = new CfgModule("root");
    final NodeVariable first = new NodeVariable("first", DataType.BOOLEAN);
    root.declareVariable(first);

    final CfgProcess child = new CfgProcess();
    root.add(child);
    final NodeVariable second = new NodeVariable("second", DataType.BOOLEAN);
    child.declareVariable(second);

    final NodeVariable third = new NodeVariable("third", DataType.BOOLEAN);

    CfgDeclarationStatement firstContainer = CfgUtils.getVariableContainer(root, first.getName());
    if (firstContainer != root) {
      Assert.fail();
    }

    CfgDeclarationStatement secondContainer = CfgUtils.getVariableContainer(root, second.getName());
    if (secondContainer != child) {
      Assert.fail();
    }

    CfgDeclarationStatement thirdContainer = CfgUtils.getVariableContainer(root, third.getName());
    if (thirdContainer != null) {
      Assert.fail();
    }
  }
}
