/*
 * Copyright 2014 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.model.efsm;

import ru.ispras.retrascope.SolverUtilTest;
import ru.ispras.retrascope.model.efsm.examples.B04;
import ru.ispras.retrascope.util.Log;

import org.junit.Test;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;

/**
 * @author <a href="mailto:lebedev@ispras.ru">Mikhail Lebedev</a>
 */
public class ExampleEfsmProductTestCase extends SolverUtilTest {

  @Test
  public void testProduct() {
    EfsmModel model1 = B04.EFSM_MODEL;
    EfsmModel model2 = B04.EFSM_MODEL;
    
    EfsmModule module1 = model1.getModules().get(0);
    EfsmModule module2 = model2.getModules().get(0);
    
    Efsm b04 = module1.getNonInitialEfsms().iterator().next();
    Efsm simpleEfsm = module2.getNonInitialEfsms().iterator().next();

    Log.printInfo(b04.getName());
    Log.printInfo("States: " + b04.getStates().size());
    Log.printInfo(b04.getStates().toString());
    Log.printInfo("Transitions: " + b04.getTransitions().size());
    Log.printInfo(b04.getTransitions().toString());
    Log.printInfo("");

    Log.printInfo(simpleEfsm.getName());
    Log.printInfo("States: " + simpleEfsm.getStates().size());
    Log.printInfo(simpleEfsm.getStates().toString());
    Log.printInfo("Transitions: " + simpleEfsm.getTransitions().size());
    Log.printInfo(simpleEfsm.getTransitions().toString());
    Log.printInfo("");

    Efsm product = EfsmUtils.product(b04, simpleEfsm);

    assertNotNull(product);
    assertEquals(product.getStates().size(),
        b04.getStates().size() * simpleEfsm.getStates().size());

    Log.printInfo(product.getName());
    Log.printInfo("States: " + product.getStates().size());
    Log.printInfo(product.getStates().toString());
    Log.printInfo("Transitions: " + product.getTransitions().size());
    for (EfsmTransition item : product.getTransitions()) {
      Log.printInfo(item.toString());
    }
    Log.printInfo("Initial state: " + product.getInitialState().toString());
    Log.printInfo("");

    Efsm product2 = EfsmUtils.product(simpleEfsm, b04);

    assertNotNull(product2);
    assertEquals(product2.getStates().size(),
        simpleEfsm.getStates().size() * b04.getStates().size());

    Log.printInfo(product2.getName());
    Log.printInfo("States: " + product2.getStates().size());
    Log.printInfo("Transitions: " + product2.getTransitions().size());
    Log.printInfo("Initial state: " + product2.getInitialState().toString());
    Log.printInfo("");

    Efsm product3 = EfsmUtils.product(simpleEfsm, simpleEfsm, simpleEfsm);

    assertNotNull(product3);
    assertEquals(product3.getStates().size(),
        simpleEfsm.getStates().size() * simpleEfsm.getStates().size()
            * simpleEfsm.getStates().size());

    Log.printInfo(product3.getName());
    Log.printInfo("States: " + product3.getStates().size());
    Log.printInfo("Transitions: " + product3.getTransitions().size());
    Log.printInfo("Initial state: " + product3.getInitialState().toString());
    Log.printInfo("");
  }

}
