/*
 * Copyright 2014 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.model.efsm.examples;

import ru.ispras.fortress.data.DataType;
import ru.ispras.fortress.expression.Node;
import ru.ispras.fortress.expression.NodeOperation;
import ru.ispras.fortress.expression.NodeValue;
import ru.ispras.fortress.expression.NodeVariable;
import ru.ispras.fortress.expression.StandardOperation;
import ru.ispras.fortress.solver.constraint.Formulas;
import ru.ispras.retrascope.model.basis.Assignment;
import ru.ispras.retrascope.model.basis.Event;
import ru.ispras.retrascope.model.basis.EventType;
import ru.ispras.retrascope.model.basis.MetaInfoType;
import ru.ispras.retrascope.model.basis.MetaInfoValue;
import ru.ispras.retrascope.model.basis.RangedVariable;
import ru.ispras.retrascope.model.efsm.Action;
import ru.ispras.retrascope.model.efsm.Efsm;
import ru.ispras.retrascope.model.efsm.EfsmModel;
import ru.ispras.retrascope.model.efsm.EfsmModule;
import ru.ispras.retrascope.model.efsm.EfsmState;
import ru.ispras.retrascope.model.efsm.Guard;
import ru.ispras.retrascope.model.efsm.GuardedAction;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

/**
 * The test class which contains a manually created ITC'99 b04 representation.
 *
 * @author <a href="mailto:i.melnichenko@deltasolutions.ru">Igor Melnichenko</a>
 */
public final class B04 {
  /**
   * An EFSM model which contains a manually created b04 EFSM representation.
   */
  public static final EfsmModel EFSM_MODEL = new EfsmModel();
//TODO add transitions and reset guarded action
  static {
    EfsmModule efsmModule = new EfsmModule("b04");

    NodeVariable restart = ExampleEfsmUtils.declareInput(efsmModule, "RESTART", "bit");
    NodeVariable average = ExampleEfsmUtils.declareInput(efsmModule, "AVERAGE", "bit");
    NodeVariable enable = ExampleEfsmUtils.declareInput(efsmModule, "ENABLE", "bit");
    NodeVariable dataIn =
        ExampleEfsmUtils.declareInput(efsmModule, "DATA_IN", "integer range 127 downto -128");

    NodeVariable dataOut =
        ExampleEfsmUtils.declareOutput(efsmModule, "DATA_OUT", "integer range 127 downto -128");

    NodeVariable clock = ExampleEfsmUtils.declareInput(efsmModule, "CLOCK", "bit");

    // TODO: uncomment when variable will be used somewhere
    /*NodeVariable reset = ExampleEfsmUtils.declareInput(efsmModule, "RESET", "bit");*/

    Efsm efsm = new Efsm(efsmModule, "EFSM1");
    efsm.addMetaInfo(MetaInfoType.HDL_LANG, MetaInfoValue.LANG_VHDL);

    NodeVariable stato =
        ExampleEfsmUtils.declareRegister(efsm, "stato", "integer range 2 downto 0");
    NodeVariable rmax =
        ExampleEfsmUtils.declareRegister(efsm, "RMAX", "integer range 127 downto -128");
    NodeVariable rmin =
        ExampleEfsmUtils.declareRegister(efsm, "RMIN", "integer range 127 downto -128");
    NodeVariable rlast =
        ExampleEfsmUtils.declareRegister(efsm, "RLAST", "integer range 127 downto -128");
    NodeVariable reg1 =
        ExampleEfsmUtils.declareRegister(efsm, "REG1", "integer range 127 downto -128");
    NodeVariable reg2 =
        ExampleEfsmUtils.declareRegister(efsm, "REG2", "integer range 127 downto -128");
    NodeVariable reg3 =
        ExampleEfsmUtils.declareRegister(efsm, "REG3", "integer range 127 downto -128");
    NodeVariable reg4 =
        ExampleEfsmUtils.declareRegister(efsm, "REG4", "integer range 127 downto -128");
    NodeVariable regd =
        ExampleEfsmUtils.declareRegister(efsm, "REGD", "integer range 127 downto -128");
    NodeVariable temp = ExampleEfsmUtils.declareRegister(efsm, "temp", "integer");
    NodeVariable res = ExampleEfsmUtils.declareRegister(efsm, "RES", "bit");
    NodeVariable ave = ExampleEfsmUtils.declareRegister(efsm, "AVE", "bit");
    NodeVariable ena = ExampleEfsmUtils.declareRegister(efsm, "ENA", "bit");

    EfsmState stateA = efsm.createState(ExampleEfsmUtils.constructEquality(stato, "0"));
    EfsmState stateB = efsm.createState(ExampleEfsmUtils.constructEquality(stato, "1"));
    EfsmState stateC = efsm.createState(ExampleEfsmUtils.constructEquality(stato, "2"));

    efsm.setInitialState(stateC);

    Node guardPredicate17 = new NodeOperation(StandardOperation.GREATER, dataIn, rmax);

    Formulas formulas = new Formulas();
    formulas.add(new NodeOperation(StandardOperation.LESS, dataIn, rmin));
    formulas.add(new NodeOperation(StandardOperation.LESSEQ, dataIn, rmax));
    Node guardPredicate18 = formulas.asSingleExpr();

    Node guardPredicate15 = new NodeOperation(StandardOperation.GREATEREQ,
        new NodeOperation(StandardOperation.ADD, dataIn, reg4),
        new NodeValue(DataType.INTEGER.valueOf("0", 10)));

    Node guardPredicate16 = new NodeOperation(StandardOperation.LESS,
        new NodeOperation(StandardOperation.ADD, dataIn,reg4),
        new NodeValue(DataType.INTEGER.valueOf("0", 10)));

    Node guardPredicate6 = ExampleEfsmUtils.constructEquality(enable, "1");

    Node guardPredicate7 = ExampleEfsmUtils.constructEquality(enable, "0");

    Node guardPredicate9 = ExampleEfsmUtils.constructEquality(restart, "1");

    formulas = new Formulas();
    formulas.add(new NodeOperation(StandardOperation.NOTEQ, restart,
        new NodeValue(DataType.BIT_VECTOR(1).valueOf("1", 10))));
    formulas.add(new NodeOperation(StandardOperation.NOTEQ, enable,
        new NodeValue(DataType.BIT_VECTOR(1).valueOf("1", 10))));
    Node guardPredicate8 = formulas.asSingleExpr();

    formulas = new Formulas();
    formulas.add(new NodeOperation(StandardOperation.NOTEQ, restart,
        new NodeValue(DataType.BIT_VECTOR(1).valueOf("1", 10))));
    formulas.add(ExampleEfsmUtils.constructEquality(enable, "1"));
    Node guardPredicate10 = formulas.asSingleExpr();

    Node guardPredicate11 = ExampleEfsmUtils.constructEquality(average, "1");

    Node guardPredicate12 = new NodeOperation(StandardOperation.NOTEQ, average,
        new NodeValue(DataType.BIT_VECTOR(1).valueOf("1", 10)));

    Node guardPredicate13 = new NodeOperation(StandardOperation.GREATEREQ,
        new NodeOperation(StandardOperation.ADD, rmax, rmin),
        new NodeValue(DataType.INTEGER.valueOf("0", 10)));

    Node guardPredicate14 = new NodeOperation(StandardOperation.LESS,
        new NodeOperation(StandardOperation.ADD, rmax, rmin),
        new NodeValue(DataType.INTEGER.valueOf("0", 10)));

    List<Assignment> statementsB = Arrays.asList(new Assignment(res, restart),
        new Assignment(ena, enable), new Assignment(ave, average));

    List<Assignment> statementsD = Arrays.asList(new Assignment(rmax, dataIn),
        new Assignment(rmin, dataIn),
        new Assignment(reg1, ExampleEfsmUtils.constructValue(reg1, "0")),
        new Assignment(reg2, ExampleEfsmUtils.constructValue(reg2, "0")),
        new Assignment(reg3, ExampleEfsmUtils.constructValue(reg3, "0")),
        new Assignment(reg4, ExampleEfsmUtils.constructValue(reg4, "0")),
        new Assignment(rlast, ExampleEfsmUtils.constructValue(rlast, "0")),
        new Assignment(dataOut, ExampleEfsmUtils.constructValue(dataOut, "0")));

    List<Assignment> statementsE = Collections.singletonList(new Assignment(rlast, dataIn));

    List<Assignment> statementsG = new ArrayList<>(2);
    statementsG.add(new Assignment(regd,
        new NodeOperation(StandardOperation.MOD,
            new NodeOperation(StandardOperation.ADD, rmax, rmin),
            new NodeValue(DataType.INTEGER.valueOf("128", 10)))));
    statementsG.add(new Assignment(temp,
        new NodeOperation(StandardOperation.ADD, rmax, rmin)));

    List<Assignment> statementsJ = Collections.singletonList(new Assignment(dataOut,
        new NodeOperation(StandardOperation.DIV, regd,
            new NodeValue(DataType.INTEGER.valueOf("2", 10)))));

    List<Assignment> statementsK = Collections.singletonList(new Assignment(dataOut,
        new NodeOperation(StandardOperation.MINUS, new NodeOperation(StandardOperation.DIV,
            new NodeOperation(StandardOperation.MINUS, regd),
            new NodeValue(DataType.INTEGER.valueOf("2", 10))))));

    List<Assignment> statementsH = Collections.singletonList(new Assignment(dataOut, reg4));

    List<Assignment> statementsI = new ArrayList<>(2);
    statementsI.add(new Assignment(regd,
        new NodeOperation(StandardOperation.MOD,
            new NodeOperation(StandardOperation.ADD, dataIn, reg4),
            new NodeValue(DataType.INTEGER.valueOf("128", 10)))));
    statementsI.add(new Assignment(temp,
        new NodeOperation(StandardOperation.ADD, dataIn, reg4)));

    List<Assignment> statementsF = Collections.singletonList(new Assignment(dataOut, rlast));

    List<Assignment> statementsN = Collections.singletonList(new Assignment(rmax, dataIn));

    List<Assignment> statementsO = Collections.singletonList(new Assignment(rmin, dataIn));

    List<Assignment> statementsP = Arrays.asList(new Assignment(reg4, reg3),
        new Assignment(reg3, reg2), new Assignment(reg2, reg1), new Assignment(reg1, dataIn));

    Event clockEvent = new Event(new RangedVariable(clock), EventType.POSITIVE_EDGE);

    {
      List<Node> guardPredicates1 = Arrays.asList(guardPredicate6, guardPredicate10,
          guardPredicate12, guardPredicate15, guardPredicate18);
      List<Assignment> assignments1 = new ArrayList<>(statementsB);
      assignments1.addAll(statementsE);
      assignments1.addAll(statementsI);
      assignments1.addAll(statementsJ);
      assignments1.addAll(statementsO);
      assignments1.addAll(statementsP);

      GuardedAction guardedAction1 =
          new GuardedAction(new Guard(guardPredicates1), new Action(assignments1));
      guardedAction1.addClock(clockEvent);

      efsm.createTransition(stateC, stateC, guardedAction1);
    }

    {
      List<Node> guardPredicates2 =
          Arrays.asList(guardPredicate7, guardPredicate8, guardPredicate17);
      List<Assignment> assignments2 = new ArrayList<>(statementsB);
      assignments2.addAll(statementsF);
      assignments2.addAll(statementsN);
      assignments2.addAll(statementsP);

      GuardedAction guardedAction2 =
          new GuardedAction(new Guard(guardPredicates2), new Action(assignments2));
      guardedAction2.addClock(clockEvent);

      efsm.createTransition(stateC, stateC, guardedAction2);
    }

    {
      List<Node> guardPredicates3 =
          Arrays.asList(guardPredicate7, guardPredicate9, guardPredicate13, guardPredicate17);
      List<Assignment> assignments3 = new ArrayList<>(statementsB);
      assignments3.addAll(statementsG);
      assignments3.addAll(statementsJ);
      assignments3.addAll(statementsN);
      assignments3.addAll(statementsP);

      GuardedAction guardedAction3 =
          new GuardedAction(new Guard(guardPredicates3), new Action(assignments3));
      guardedAction3.addClock(clockEvent);

      efsm.createTransition(stateC, stateC, guardedAction3);
    }

    {
      List<Node> guardPredicates4 =
          Arrays.asList(guardPredicate7, guardPredicate9, guardPredicate14, guardPredicate17);
      List<Assignment> assignments4 = new ArrayList<>(statementsB);
      assignments4.addAll(statementsG);
      assignments4.addAll(statementsK);
      assignments4.addAll(statementsN);
      assignments4.addAll(statementsP);

      GuardedAction guardedAction4 =
          new GuardedAction(new Guard(guardPredicates4), new Action(assignments4));
      guardedAction4.addClock(clockEvent);

      efsm.createTransition(stateC, stateC, guardedAction4);
    }

    {
      List<Node> guardPredicates8 =
          Arrays.asList(guardPredicate7, guardPredicate8, guardPredicate18);
      List<Assignment> assignments8 = new ArrayList<>(statementsB);
      assignments8.addAll(statementsF);
      assignments8.addAll(statementsO);
      assignments8.addAll(statementsP);

      GuardedAction guardedAction8 =
          new GuardedAction(new Guard(guardPredicates8), new Action(assignments8));
      guardedAction8.addClock(clockEvent);

      efsm.createTransition(stateC, stateC, guardedAction8);
    }

    {
      List<Node> guardPredicates9 =
          Arrays.asList(guardPredicate7, guardPredicate9, guardPredicate13, guardPredicate18);
      List<Assignment> assignments9 = new ArrayList<>(statementsB);
      assignments9.addAll(statementsG);
      assignments9.addAll(statementsJ);
      assignments9.addAll(statementsO);
      assignments9.addAll(statementsP);
      GuardedAction guardedAction9 =
          new GuardedAction(new Guard(guardPredicates9), new Action(assignments9));
      guardedAction9.addClock(clockEvent);
      efsm.createTransition(stateC, stateC, guardedAction9);
    }

    {
      List<Node> guardPredicates10 =
          Arrays.asList(guardPredicate7, guardPredicate9, guardPredicate14, guardPredicate18);
      List<Assignment> assignments10 = new ArrayList<>(statementsB);
      assignments10.addAll(statementsG);
      assignments10.addAll(statementsK);
      assignments10.addAll(statementsO);
      assignments10.addAll(statementsP);
      GuardedAction guardedAction10 =
          new GuardedAction(new Guard(guardPredicates10), new Action(assignments10));
      guardedAction10.addClock(clockEvent);
      efsm.createTransition(stateC, stateC, guardedAction10);
    }

    {
      GuardedAction guardedAction22 =
          new GuardedAction(ExampleEfsmUtils.ALWAYS_TRUE_GUARD, new Action(statementsB));
      guardedAction22.addClock(clockEvent);

      efsm.createTransition(stateA, stateB, guardedAction22);
    }

    {
      List<Assignment> assignments23 = new ArrayList<>(statementsB);
      assignments23.addAll(statementsD);

      GuardedAction guardedAction23 =
          new GuardedAction(ExampleEfsmUtils.ALWAYS_TRUE_GUARD, new Action(assignments23));
      guardedAction23.addClock(clockEvent);

      efsm.createTransition(stateB, stateC, guardedAction23);
    }

    {
      List<Node> guardPredicates25 =
          Arrays.asList(guardPredicate6, guardPredicate9, guardPredicate13, guardPredicate17);
      List<Assignment> assignments25 = new ArrayList<>(statementsB);
      assignments25.addAll(statementsE);
      assignments25.addAll(statementsG);
      assignments25.addAll(statementsJ);
      assignments25.addAll(statementsN);
      assignments25.addAll(statementsP);
      GuardedAction guardedAction25 =
          new GuardedAction(new Guard(guardPredicates25), new Action(assignments25));
      guardedAction25.addClock(clockEvent);
      efsm.createTransition(stateC, stateC, guardedAction25);
    }

    {
      List<Node> guardPredicates26 =
          Arrays.asList(guardPredicate6, guardPredicate9, guardPredicate14, guardPredicate17);
      List<Assignment> assignments26 = new ArrayList<>(statementsB);
      assignments26.addAll(statementsE);
      assignments26.addAll(statementsG);
      assignments26.addAll(statementsK);
      assignments26.addAll(statementsN);
      assignments26.addAll(statementsP);
      GuardedAction guardedAction26 =
          new GuardedAction(new Guard(guardPredicates26), new Action(assignments26));
      guardedAction26.addClock(clockEvent);
      efsm.createTransition(stateC, stateC, guardedAction26);
    }

    {
      Guard guard27 = new Guard(Arrays.asList(guardPredicate6, guardPredicate10, guardPredicate11,
          guardPredicate17));
      List<Assignment> assignments27 = new ArrayList<>(statementsB);
      assignments27.addAll(statementsE);
      assignments27.addAll(statementsH);
      assignments27.addAll(statementsN);
      assignments27.addAll(statementsP);

      GuardedAction guardedAction27 = new GuardedAction(guard27, new Action(assignments27));
      guardedAction27.addClock(clockEvent);

      efsm.createTransition(stateC, stateC, guardedAction27);
    }

    {
      List<Node> guardPredicates28 = Arrays.asList(guardPredicate6, guardPredicate10,
          guardPredicate12, guardPredicate15, guardPredicate17);
      List<Assignment> assignments28 = new ArrayList<>(statementsB);
      assignments28.addAll(statementsE);
      assignments28.addAll(statementsI);
      assignments28.addAll(statementsJ);
      assignments28.addAll(statementsN);
      assignments28.addAll(statementsP);

      GuardedAction guardedAction28 =
          new GuardedAction(new Guard(guardPredicates28), new Action(assignments28));
      guardedAction28.addClock(clockEvent);

      efsm.createTransition(stateC, stateC, guardedAction28);
    }

    {
      List<Node> guardPredicates29 = Arrays.asList(guardPredicate6, guardPredicate10,
          guardPredicate12, guardPredicate16, guardPredicate17);
      List<Assignment> assignments29 = new ArrayList<>(statementsB);
      assignments29.addAll(statementsE);
      assignments29.addAll(statementsI);
      assignments29.addAll(statementsK);
      assignments29.addAll(statementsN);
      assignments29.addAll(statementsP);

      GuardedAction guardedAction29 =
          new GuardedAction(new Guard(guardPredicates29), new Action(assignments29));
      guardedAction29.addClock(clockEvent);

      efsm.createTransition(stateC, stateC, guardedAction29);
    }

    {
      List<Node> guardPredicates31 =
          Arrays.asList(guardPredicate6, guardPredicate9, guardPredicate13, guardPredicate18);
      List<Assignment> assignments31 = new ArrayList<>(statementsB);
      assignments31.addAll(statementsE);
      assignments31.addAll(statementsG);
      assignments31.addAll(statementsJ);
      assignments31.addAll(statementsO);
      assignments31.addAll(statementsP);

      GuardedAction guardedAction31 =
          new GuardedAction(new Guard(guardPredicates31), new Action(assignments31));
      guardedAction31.addClock(clockEvent);

      efsm.createTransition(stateC, stateC, guardedAction31);
    }

    {
      List<Node> guardPredicates32 =
          Arrays.asList(guardPredicate6, guardPredicate9, guardPredicate14, guardPredicate18);
      List<Assignment> assignments32 = new ArrayList<>(statementsB);
      assignments32.addAll(statementsE);
      assignments32.addAll(statementsG);
      assignments32.addAll(statementsK);
      assignments32.addAll(statementsO);
      assignments32.addAll(statementsP);

      GuardedAction guardedAction32 =
          new GuardedAction(new Guard(guardPredicates32), new Action(assignments32));
      guardedAction32.addClock(clockEvent);

      efsm.createTransition(stateC, stateC, guardedAction32);
    }

    {
      List<Node> guardPredicates33 =
          Arrays.asList(guardPredicate6, guardPredicate10, guardPredicate11, guardPredicate18);
      List<Assignment> assignments33 = new ArrayList<>(statementsB);
      assignments33.addAll(statementsE);
      assignments33.addAll(statementsH);
      assignments33.addAll(statementsO);
      assignments33.addAll(statementsP);

      GuardedAction guardedAction33 =
          new GuardedAction(new Guard(guardPredicates33), new Action(assignments33));
      guardedAction33.addClock(clockEvent);

      efsm.createTransition(stateC, stateC, guardedAction33);
    }

    // TODO: uncomment when event list will be used somewhere
    /*Event resetEvent = new Event(new RangedVariable(reset), EventType.POSITIVE_EDGE);
    ArrayList<Event> resetEventList = new ArrayList<>(1);
    resetEventList.add(resetEvent);*/

    efsmModule.addNonInitialEfsm(efsm);
    B04.EFSM_MODEL.addModule(efsmModule);
  }

  private B04() {
  }
}
