/*
 * Copyright 2014 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.retrascope.model.efsm.examples;

import ru.ispras.fortress.data.DataType;
import ru.ispras.fortress.expression.Node;
import ru.ispras.fortress.expression.NodeOperation;
import ru.ispras.fortress.expression.NodeValue;
import ru.ispras.fortress.expression.NodeVariable;
import ru.ispras.fortress.expression.StandardOperation;
import ru.ispras.retrascope.model.basis.Assignment;
import ru.ispras.retrascope.model.basis.Event;
import ru.ispras.retrascope.model.basis.EventType;
import ru.ispras.retrascope.model.basis.MetaInfoType;
import ru.ispras.retrascope.model.basis.MetaInfoValue;
import ru.ispras.retrascope.model.basis.RangedVariable;
import ru.ispras.retrascope.model.efsm.Action;
import ru.ispras.retrascope.model.efsm.Efsm;
import ru.ispras.retrascope.model.efsm.EfsmModel;
import ru.ispras.retrascope.model.efsm.EfsmModule;
import ru.ispras.retrascope.model.efsm.EfsmState;
import ru.ispras.retrascope.model.efsm.Guard;
import ru.ispras.retrascope.model.efsm.GuardedAction;
import ru.ispras.retrascope.util.NodeUtils;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

/**
 * The test class which contains a manually created ITC'99 b13 representation.
 *
 * @author <a href="mailto:lebedev@ispras.ru">Mikhail Lebedev</a>
 * @author <a href="mailto:i.melnichenko@deltasolutions.ru">Igor Melnichenko</a>
 */
public final class B13 {
  /**
   * An EFSM model which contains a manually created b13 EFSM representation.
   */
  public static final EfsmModel EFSM_MODEL = new EfsmModel();

  private static final String GP001 = "0";
  private static final String GP010 = "1";
  private static final String GP011 = "2";
  private static final String GP100 = "3";
  private static final String GP100w = "4";
  private static final String GP101 = "5";
  private static final String GP110 = "6";
  private static final String GP111 = "7";

  private static final String GP01 = "0";
  private static final String GP10 = "1";
  private static final String GP11 = "2";
  private static final String GP11w = "3";

  private static final String START_BIT = "0";
  private static final String STOP_BIT = "1";
  private static final String BIT0 = "2";
  private static final String BIT1 = "3";
  private static final String BIT2 = "4";
  private static final String BIT3 = "5";
  private static final String BIT4 = "6";
  private static final String BIT5 = "7";
  private static final String BIT6 = "8";
  private static final String BIT7 = "9";

  private static final String G_IDLE = "0";
  private static final String G_LOAD = "1";
  private static final String G_SEND = "2";
  private static final String G_WAIT_END = "3";

//TODO check transitions and reset guarded action
  static {
    EfsmModule efsmModule = new EfsmModule("b13");

    NodeVariable reset = ExampleEfsmUtils.declareInput(efsmModule, "reset", "bit");
    NodeVariable eoc = ExampleEfsmUtils.declareInput(efsmModule, "eoc", "bit");

    NodeVariable soc = ExampleEfsmUtils.declareOutput(efsmModule, "soc", "bit");
    NodeVariable loadDato = ExampleEfsmUtils.declareOutput(efsmModule, "load_dato", "bit");
    NodeVariable addMpx2 = ExampleEfsmUtils.declareOutput(efsmModule, "add_mpx2", "bit");

    NodeVariable canale = ExampleEfsmUtils.declareOutput(efsmModule, "canale",
        "integer range 8 downto 0");
    NodeVariable muxEn = ExampleEfsmUtils.declareOutput(efsmModule, "mux_en", "bit");

    NodeVariable clock = ExampleEfsmUtils.declareInput(efsmModule, "clock", "bit");

    NodeVariable dataIn =
        ExampleEfsmUtils.declareInput(efsmModule, "data_in", "bit_vector( 7 downto 0 )");
    NodeVariable dsr = ExampleEfsmUtils.declareInput(efsmModule, "dsr", "bit");

    NodeVariable error = ExampleEfsmUtils.declareOutput(efsmModule, "error", "bit");
    NodeVariable dataOut = ExampleEfsmUtils.declareOutput(efsmModule, "data_out", "bit");

    NodeVariable s1 =
        ExampleEfsmUtils.declareRegister(efsmModule, "S1", "integer range 7 downto 0");
    NodeVariable s2 =
        ExampleEfsmUtils.declareRegister(efsmModule, "S2", "integer range 3 downto 0");

    NodeVariable mpx = ExampleEfsmUtils.declareRegister(efsmModule, "mpx", "bit");
    NodeVariable rdy = ExampleEfsmUtils.declareRegister(efsmModule, "rdy", "bit");
    NodeVariable sendData = ExampleEfsmUtils.declareRegister(efsmModule, "send_data", "bit");

    NodeVariable confirm = ExampleEfsmUtils.declareRegister(efsmModule, "confirm", "bit");
    NodeVariable shot = ExampleEfsmUtils.declareRegister(efsmModule, "shot", "bit");

    NodeVariable sendEn = ExampleEfsmUtils.declareRegister(efsmModule, "send_en", "bit");
    NodeVariable tre = ExampleEfsmUtils.declareRegister(efsmModule, "tre", "bit");
    NodeVariable outReg =
        ExampleEfsmUtils.declareRegister(efsmModule, "out_reg", "bit_vector( 7 downto 0 )");
    NodeVariable nextBit =
        ExampleEfsmUtils.declareRegister(efsmModule, "next_bit", "integer range 9 downto 0");
    NodeVariable txEnd = ExampleEfsmUtils.declareRegister(efsmModule, "tx_end", "bit");
    NodeVariable itfcState =
        ExampleEfsmUtils.declareRegister(efsmModule, "itfc_state", "integer range 3 downto 0");
    NodeVariable send = ExampleEfsmUtils.declareRegister(efsmModule, "send", "bit");
    NodeVariable load = ExampleEfsmUtils.declareRegister(efsmModule, "load", "bit");

    NodeVariable txConta =
        ExampleEfsmUtils.declareRegister(efsmModule, "tx_conta", "integer range 512 downto 0");

    Event clockEvent = new Event(new RangedVariable(clock), EventType.POSITIVE_EDGE);
    Event resetEvent = new Event(new RangedVariable(reset), EventType.POSITIVE_EDGE);

    {
      Efsm process1 = new Efsm(efsmModule, "B13_process1");
      process1.addMetaInfo(MetaInfoType.HDL_LANG, MetaInfoValue.LANG_VHDL);

      NodeVariable contaTmp =
          ExampleEfsmUtils.declareRegister(process1, "conta_tmp", "integer range 8 downto 0");

      // States
      EfsmState gp001State =
          process1.createState(NodeUtils.getEquation(s1, new NodeValue(
              DataType.INTEGER.valueOf(String.valueOf(GP001), 16))));
      EfsmState gp010State =
          process1.createState(NodeUtils.getEquation(s1, new NodeValue(
              DataType.INTEGER.valueOf(String.valueOf(GP010), 16))));
      EfsmState gp011State =
          process1.createState(NodeUtils.getEquation(s1, new NodeValue(
              DataType.INTEGER.valueOf(String.valueOf(GP011), 16))));
      EfsmState gp101State =
          process1.createState(NodeUtils.getEquation(s1, new NodeValue(
              DataType.INTEGER.valueOf(String.valueOf(GP101), 16))));
      EfsmState gp110State =
          process1.createState(NodeUtils.getEquation(s1, new NodeValue(
              DataType.INTEGER.valueOf(String.valueOf(GP110), 16))));
      EfsmState gp111State =
          process1.createState(NodeUtils.getEquation(s1, new NodeValue(
              DataType.INTEGER.valueOf(String.valueOf(GP111), 16))));
      EfsmState gp100wState =
          process1.createState(NodeUtils.getEquation(s1, new NodeValue(
              DataType.INTEGER.valueOf(String.valueOf(GP100w), 16))));
      EfsmState gp100State =
          process1.createState(NodeUtils.getEquation(s1, new NodeValue(
              DataType.INTEGER.valueOf(String.valueOf(GP100), 15))));

      process1.setInitialState(gp001State);

      // Guards
      Node p1GuardPredicate1 = ExampleEfsmUtils.constructEquality(rdy, "0");
      Node p1GuardPredicate2 = ExampleEfsmUtils.constructEquality(contaTmp, "8");
      Node p1GuardPredicate3 = ExampleEfsmUtils.constructEquality(eoc, "1");

      // Assignments
      List<Assignment> p1Statements1 = new ArrayList<>();
      p1Statements1.add(new Assignment(soc, ExampleEfsmUtils.constructValue(soc, "0")));
      p1Statements1.add(new Assignment(canale, ExampleEfsmUtils.constructValue(canale, "0")));
      p1Statements1.add(new Assignment(contaTmp, ExampleEfsmUtils.constructValue(contaTmp, "0")));
      p1Statements1.add(new Assignment(sendData, ExampleEfsmUtils.constructValue(sendData, "0")));
      p1Statements1.add(new Assignment(loadDato, ExampleEfsmUtils.constructValue(loadDato, "0")));
      p1Statements1.add(new Assignment(muxEn, ExampleEfsmUtils.constructValue(muxEn, "0")));
      p1Statements1.add(new Assignment(s1, ExampleEfsmUtils.constructValue(s1, "0")));

      List<Assignment> p1Statements2 = new ArrayList<>();
      p1Statements2.add(new Assignment(muxEn, ExampleEfsmUtils.constructValue(muxEn, "1")));
      p1Statements2.add(new Assignment(s1, ExampleEfsmUtils.constructValue(s1, "1")));

      Assignment p1Statement3 = new Assignment(s1, ExampleEfsmUtils.constructValue(s1, "2"));

      List<Assignment> p1Statements4 = new ArrayList<>();
      p1Statements4.add(ExampleEfsmUtils.constructAssignment(soc, "1"));
      p1Statements4.add(ExampleEfsmUtils.constructAssignment(s1, "5"));

      List<Assignment> p1Statements5 = new ArrayList<>();
      p1Statements5.add(ExampleEfsmUtils.constructAssignment(loadDato, "0"));
      p1Statements5.add(ExampleEfsmUtils.constructAssignment(soc, "0"));
      p1Statements5.add(new Assignment(contaTmp, new NodeOperation(StandardOperation.ADD, contaTmp,
          ExampleEfsmUtils.constructValue(contaTmp, "1"))));

      List<Assignment> p1Statements6 = new ArrayList<>();
      p1Statements6.add(ExampleEfsmUtils.constructAssignment(sendData, "1"));
      p1Statements6.add(ExampleEfsmUtils.constructAssignment(s1, "4"));

      List<Assignment> p1Statements7 =
          Collections.singletonList(ExampleEfsmUtils.constructAssignment(contaTmp, "0"));

      Assignment p1Statement8 = ExampleEfsmUtils.constructAssignment(s1, "3");

      List<Assignment> p1Statements9 = new ArrayList<>();
      p1Statements9.add(new Assignment(canale, contaTmp));
      p1Statements9.add(ExampleEfsmUtils.constructAssignment(s1, "7"));

      List<Assignment> p1Statements10 = new ArrayList<>();
      p1Statements10.add(ExampleEfsmUtils.constructAssignment(loadDato, "1"));
      p1Statements10.add(ExampleEfsmUtils.constructAssignment(muxEn, "0"));
      p1Statements10.add(ExampleEfsmUtils.constructAssignment(s1, "6"));

      Assignment p1Statement11 = ExampleEfsmUtils.constructAssignment(s1, "5");

      List<Assignment> p1Statements12 = new ArrayList<>();
      p1Statements12.add(ExampleEfsmUtils.constructAssignment(sendData, "0"));
      p1Statements12.add(ExampleEfsmUtils.constructAssignment(s1, "0"));

      Assignment p1Statement13 = ExampleEfsmUtils.constructAssignment(s1, "3");

      // Guarded actions
      Action p1Action1 = new Action(p1Statements1);
      GuardedAction p1Reset = new GuardedAction(ExampleEfsmUtils.ALWAYS_TRUE_GUARD, p1Action1);
      p1Reset.addClock(resetEvent);

      Action p1Action2 = new Action(p1Statements2);
      GuardedAction p1ga2 = new GuardedAction(ExampleEfsmUtils.ALWAYS_TRUE_GUARD, p1Action2);
      p1ga2.addClock(clockEvent);

      GuardedAction p1ga3 =
          new GuardedAction(ExampleEfsmUtils.ALWAYS_TRUE_GUARD, new Action(p1Statement3));
      p1ga3.addClock(clockEvent);

      Action p1Action4 = new Action(p1Statements4);
      GuardedAction p1ga4 = new GuardedAction(ExampleEfsmUtils.ALWAYS_TRUE_GUARD, p1Action4);
      p1ga4.addClock(clockEvent);

      GuardedAction p1ga5 =
          new GuardedAction(new Guard(p1GuardPredicate3), new Action(p1Statement11));
      p1ga5.addClock(clockEvent);

      Action p1Action6 = new Action(p1Statements10);
      GuardedAction p1ga6 = new GuardedAction(new Guard(Node.NOT(p1GuardPredicate3)), p1Action6);
      p1ga6.addClock(clockEvent);

      List<Assignment> p1Action7Statements = new ArrayList<>();
      p1Action7Statements.addAll(p1Statements5);
      p1Action7Statements.addAll(p1Statements7);
      p1Action7Statements.addAll(p1Statements9);
      Action p1Action7 = new Action(p1Action7Statements);
      GuardedAction p1ga7 = new GuardedAction(new Guard(p1GuardPredicate2), p1Action7);
      p1ga7.addClock(clockEvent);

      List<Assignment> p1Action8Statements = new ArrayList<>();
      p1Action8Statements.addAll(p1Statements5);
      p1Action8Statements.addAll(p1Statements9);
      Action p1Action8 = new Action(p1Action8Statements);
      GuardedAction p1ga8 = new GuardedAction(new Guard(Node.NOT(p1GuardPredicate2)), p1Action8);
      p1ga8.addClock(clockEvent);

      Action p1Action9 = new Action(p1Statements6);
      GuardedAction p1ga9 = new GuardedAction(ExampleEfsmUtils.ALWAYS_TRUE_GUARD, p1Action9);
      p1ga9.addClock(clockEvent);

      GuardedAction p1ga10 =
          new GuardedAction(ExampleEfsmUtils.ALWAYS_TRUE_GUARD, new Action(p1Statement8));
      p1ga10.addClock(clockEvent);

      Action p1Action11 = new Action(p1Statements12);
      GuardedAction p1ga11 = new GuardedAction(new Guard(Node.NOT(p1GuardPredicate1)), p1Action11);
      p1ga11.addClock(clockEvent);

      GuardedAction p1ga12 =
          new GuardedAction(new Guard(p1GuardPredicate1), new Action(p1Statement13));
      p1ga12.addClock(clockEvent);

      // Transitions
      process1.createTransition(gp001State, gp001State, p1Reset);
      process1.createTransition(gp010State, gp001State, p1Reset);
      process1.createTransition(gp011State, gp001State, p1Reset);
      process1.createTransition(gp100State, gp001State, p1Reset);
      process1.createTransition(gp100wState, gp001State, p1Reset);
      process1.createTransition(gp101State, gp001State, p1Reset);
      process1.createTransition(gp110State, gp001State, p1Reset);
      process1.createTransition(gp111State, gp001State, p1Reset);

      process1.createTransition(gp001State, gp010State, p1ga2);
      process1.createTransition(gp010State, gp011State, p1ga3);
      process1.createTransition(gp011State, gp101State, p1ga4);
      process1.createTransition(gp101State, gp101State, p1ga5);
      process1.createTransition(gp101State, gp110State, p1ga6);
      process1.createTransition(gp110State, gp111State, p1ga7);
      process1.createTransition(gp110State, gp111State, p1ga8);
      process1.createTransition(gp111State, gp100wState, p1ga9);
      process1.createTransition(gp100wState, gp100State, p1ga10);
      process1.createTransition(gp100State, gp100State, p1ga12);
      process1.createTransition(gp100State, gp001State, p1ga11);

      efsmModule.addNonInitialEfsm(process1);
    }

    {
      Efsm process2 = new Efsm(efsmModule, "B13_process2");
      process2.addMetaInfo(MetaInfoType.HDL_LANG, MetaInfoValue.LANG_VHDL);

      // States
      EfsmState gp01State =
          process2.createState(NodeUtils.getEquation(s2, new NodeValue(
              DataType.INTEGER.valueOf(String.valueOf(GP01), 16))));
      EfsmState gp10State =
          process2.createState(NodeUtils.getEquation(s2, new NodeValue(
              DataType.INTEGER.valueOf(String.valueOf(GP10), 16))));
      EfsmState gp11State =
          process2.createState(NodeUtils.getEquation(s2, new NodeValue(
              DataType.INTEGER.valueOf(String.valueOf(GP11), 16))));
      EfsmState gp11wState =
          process2.createState(NodeUtils.getEquation(s2, new NodeValue(
              DataType.INTEGER.valueOf(String.valueOf(GP11w), 16))));

      process2.setInitialState(gp01State);

      // Guards
      Node p2GuardPredicate1 = ExampleEfsmUtils.constructEquality(confirm, "0");
      Node p2GuardPredicate2 = ExampleEfsmUtils.constructEquality(sendData, "1");
      Node p2GuardPredicate3 = ExampleEfsmUtils.constructEquality(mpx, "0");

      // Assignments
      List<Assignment> p2Statements1 = Arrays.asList(ExampleEfsmUtils.constructAssignment(rdy, "0"),
          ExampleEfsmUtils.constructAssignment(addMpx2, "0"),
          ExampleEfsmUtils.constructAssignment(mpx, "0"),
          ExampleEfsmUtils.constructAssignment(shot, "0"),
          ExampleEfsmUtils.constructAssignment(s2, "0"));

      ArrayList<Assignment> p2Statements2 = new ArrayList<>();
      p2Statements2.add(ExampleEfsmUtils.constructAssignment(shot, "1"));
      p2Statements2.add(ExampleEfsmUtils.constructAssignment(s2, "2"));

      ArrayList<Assignment> p2Statements3 = new ArrayList<>();
      p2Statements3.add(ExampleEfsmUtils.constructAssignment(shot, "0"));
      p2Statements3.add(ExampleEfsmUtils.constructAssignment(s2, "2"));

      ArrayList<Assignment> p2Statements4 = new ArrayList<>();
      p2Statements4.add(ExampleEfsmUtils.constructAssignment(rdy, "1"));
      p2Statements4.add(ExampleEfsmUtils.constructAssignment(s2, "1"));

      Assignment p2Statement5 = ExampleEfsmUtils.constructAssignment(s2, "0");

      ArrayList<Assignment> p2Statements6 = new ArrayList<>();
      p2Statements6.add(ExampleEfsmUtils.constructAssignment(addMpx2, "1"));
      p2Statements6.add(ExampleEfsmUtils.constructAssignment(mpx, "1"));
      p2Statements6.add(ExampleEfsmUtils.constructAssignment(s2, "1"));

      ArrayList<Assignment> p2Statements7 = new ArrayList<>();
      p2Statements7.add(ExampleEfsmUtils.constructAssignment(mpx, "0"));
      p2Statements7.add(ExampleEfsmUtils.constructAssignment(rdy, "0"));
      p2Statements7.add(ExampleEfsmUtils.constructAssignment(s2, "3"));

      Assignment p2Statement8 = ExampleEfsmUtils.constructAssignment(s2, "0");

      // Guarded actions
      Action p2Action1 = new Action(p2Statements1);
      GuardedAction p2Reset = new GuardedAction(ExampleEfsmUtils.ALWAYS_TRUE_GUARD, p2Action1);
      p2Reset.addClock(resetEvent);

      Action p2Action2 = new Action(p2Statements4);
      GuardedAction p2ga2 = new GuardedAction(new Guard(p2GuardPredicate2), p2Action2);
      p2ga2.addClock(clockEvent);

      GuardedAction p2ga3 =
          new GuardedAction(new Guard(Node.NOT(p2GuardPredicate2)),
              new Action(p2Statement5));
      p2ga3.addClock(clockEvent);

      Action p2Action4 = new Action(p2Statements2);
      GuardedAction p2ga4 = new GuardedAction(ExampleEfsmUtils.ALWAYS_TRUE_GUARD, p2Action4);
      p2ga4.addClock(clockEvent);

      Action p2Action5 = new Action(p2Statements3);
      GuardedAction p2ga5 = new GuardedAction(new Guard(p2GuardPredicate1), p2Action5);
      p2ga5.addClock(clockEvent);

      ArrayList<Node> p2ga6Nodes = new ArrayList<>();
      p2ga6Nodes.add(Node.NOT(p2GuardPredicate1));
      p2ga6Nodes.add(p2GuardPredicate3);
      Action p2Action6 = new Action(p2Statements6);
      GuardedAction p2ga6 = new GuardedAction(new Guard(p2ga6Nodes), p2Action6);
      p2ga6.addClock(clockEvent);

      ArrayList<Node> p2ga7Nodes = new ArrayList<>();
      p2ga7Nodes.add(Node.NOT(p2GuardPredicate1));
      p2ga7Nodes.add(Node.NOT(p2GuardPredicate3));
      Action p2Action7 = new Action(p2Statements7);
      GuardedAction p2ga7 = new GuardedAction(new Guard(p2ga7Nodes), p2Action7);
      p2ga7.addClock(clockEvent);

      GuardedAction p2ga8 =
          new GuardedAction(ExampleEfsmUtils.ALWAYS_TRUE_GUARD, new Action(p2Statement8));
      p2ga8.addClock(clockEvent);

      // Transitions
      process2.createTransition(gp01State, gp01State, p2Reset);
      process2.createTransition(gp10State, gp01State, p2Reset);
      process2.createTransition(gp11State, gp01State, p2Reset);
      process2.createTransition(gp11wState, gp01State, p2Reset);

      process2.createTransition(gp01State, gp10State, p2ga2);
      process2.createTransition(gp01State, gp01State, p2ga3);
      process2.createTransition(gp10State, gp11State, p2ga4);
      process2.createTransition(gp11State, gp11State, p2ga5);
      process2.createTransition(gp11State, gp10State, p2ga6);
      process2.createTransition(gp11State, gp11wState, p2ga7);
      process2.createTransition(gp11wState, gp01State, p2ga8);

      efsmModule.addNonInitialEfsm(process2);
    }

    {
      Efsm process3 = new Efsm(efsmModule, "B13_process3");
      process3.addMetaInfo(MetaInfoType.HDL_LANG, MetaInfoValue.LANG_VHDL);

      // States
      EfsmState idleState =
          process3.createState(ExampleEfsmUtils.constructEquality(itfcState, G_IDLE));
      EfsmState loadState =
          process3.createState(ExampleEfsmUtils.constructEquality(itfcState, G_LOAD));
      EfsmState sendState =
          process3.createState(ExampleEfsmUtils.constructEquality(itfcState, G_SEND));
      EfsmState waitEndState =
          process3.createState(ExampleEfsmUtils.constructEquality(itfcState, G_WAIT_END));

      process3.setInitialState(idleState);

      // Guards
      Node p3GuardPredicate1 = ExampleEfsmUtils.constructEquality(shot, "1");
      Node p3GuardPredicate2 = ExampleEfsmUtils.constructEquality(txEnd, "1");

      // Assignments
      List<Assignment> p3Statements1 = new ArrayList<>();
      p3Statements1.add(ExampleEfsmUtils.constructAssignment(load, "0"));
      p3Statements1.add(ExampleEfsmUtils.constructAssignment(send, "0"));
      p3Statements1.add(ExampleEfsmUtils.constructAssignment(confirm, "0"));

      List<Assignment> p3Statements2 = new ArrayList<>();
      p3Statements2.add(ExampleEfsmUtils.constructAssignment(load, "0"));
      p3Statements2.add(ExampleEfsmUtils.constructAssignment(send, "1"));

      List<Assignment> p3Statements3 = new ArrayList<>();
      p3Statements3.add(ExampleEfsmUtils.constructAssignment(send, "0"));

      List<Assignment> p3Statements4 = new ArrayList<>();
      p3Statements4.add(ExampleEfsmUtils.constructAssignment(load, "1"));
      p3Statements4.add(ExampleEfsmUtils.constructAssignment(confirm, "0"));

      List<Assignment> p3Statements5 = new ArrayList<>();
      p3Statements5.add(ExampleEfsmUtils.constructAssignment(confirm, "0"));

      List<Assignment> p3Statements6 = new ArrayList<>();
      p3Statements6.add(ExampleEfsmUtils.constructAssignment(confirm, "1"));

      // Guarded actions
      Action p3Action1 = new Action(p3Statements1);
      GuardedAction p3Reset = new GuardedAction(ExampleEfsmUtils.ALWAYS_TRUE_GUARD, p3Action1);
      p3Reset.addClock(resetEvent);

      Action p3Action2 = new Action(p3Statements4);
      GuardedAction p3ga2 = new GuardedAction(new Guard(p3GuardPredicate1), p3Action2);
      p3ga2.addClock(clockEvent);

      Action p3Action3 = new Action(p3Statements5);
      GuardedAction p3ga3 = new GuardedAction(new Guard(Node.NOT(p3GuardPredicate1)), p3Action3);
      p3ga3.addClock(clockEvent);

      Action p3Action4 = new Action(p3Statements2);
      GuardedAction p3ga4 = new GuardedAction(ExampleEfsmUtils.ALWAYS_TRUE_GUARD, p3Action4);
      p3ga4.addClock(clockEvent);

      Action p3Action5 = new Action(p3Statements3);
      GuardedAction p3ga5 = new GuardedAction(ExampleEfsmUtils.ALWAYS_TRUE_GUARD, p3Action5);
      p3ga5.addClock(clockEvent);

      Action p3Action6 = new Action(p3Statements6);
      GuardedAction p3ga6 = new GuardedAction(new Guard(p3GuardPredicate2), p3Action6);
      p3ga6.addClock(clockEvent);

      // Transitions
      process3.createTransition(idleState, idleState, p3Reset);
      process3.createTransition(loadState, idleState, p3Reset);
      process3.createTransition(sendState, idleState, p3Reset);
      process3.createTransition(waitEndState, idleState, p3Reset);

      process3.createTransition(idleState, loadState, p3ga2);
      process3.createTransition(idleState, idleState, p3ga3);
      process3.createTransition(loadState, sendState, p3ga4);
      process3.createTransition(sendState, waitEndState, p3ga5);
      process3.createTransition(waitEndState, idleState, p3ga6);

      efsmModule.addNonInitialEfsm(process3);
    }

    {
      Efsm process4 = new Efsm(efsmModule, "B13_process4");
      process4.addMetaInfo(MetaInfoType.HDL_LANG, MetaInfoValue.LANG_VHDL);

      EfsmState p4State1 = process4.createState(ExampleEfsmUtils.constructEquality(tre, "0"));
      EfsmState p4State2 = process4.createState(ExampleEfsmUtils.constructEquality(tre, "1"));

      process4.setInitialState(p4State1);

      // Guards
      Node p4GuardPredicate1 = ExampleEfsmUtils.constructEquality(txEnd, "1");
      Node p4GuardPredicate2 = ExampleEfsmUtils.constructEquality(load, "1");
      Node p4GuardPredicate4 = ExampleEfsmUtils.constructEquality(send, "1");
      Node p4GuardPredicate5 = ExampleEfsmUtils.constructEquality(dsr, "0");

      // Assignments
      ArrayList<Assignment> p4Statements1 = new ArrayList<>();
      p4Statements1.add(ExampleEfsmUtils.constructAssignment(sendEn, "0"));
      p4Statements1.add(ExampleEfsmUtils.constructAssignment(outReg, "0"));
      p4Statements1.add(ExampleEfsmUtils.constructAssignment(tre, "0"));
      p4Statements1.add(ExampleEfsmUtils.constructAssignment(error, "0"));

      Assignment p4Statement2 = ExampleEfsmUtils.constructAssignment(sendEn, "0");
      Assignment p4Statement3 = ExampleEfsmUtils.constructAssignment(tre, "1");
      Assignment p4Statement4 = new Assignment(outReg, dataIn);
      Assignment p4Statement6 = ExampleEfsmUtils.constructAssignment(error, "0");
      Assignment p4Statement7 = ExampleEfsmUtils.constructAssignment(error, "1");
      Assignment p4Statement8 = ExampleEfsmUtils.constructAssignment(sendEn, "1");

      // Guarded actions
      Action p4Action1 = new Action(p4Statements1);
      GuardedAction p4Reset = new GuardedAction(ExampleEfsmUtils.ALWAYS_TRUE_GUARD, p4Action1);
      p4Reset.addClock(resetEvent);

      Action p4Action3 = new Action(p4Statement7);
      List<Node> guard3Nodes = new ArrayList<>();
      guard3Nodes.add(Node.NOT(p4GuardPredicate1));
      guard3Nodes.add(Node.NOT(p4GuardPredicate2));
      guard3Nodes.add(p4GuardPredicate4);
      GuardedAction p4ga31 = new GuardedAction(new Guard(guard3Nodes), p4Action3);
      p4ga31.addClock(clockEvent);

      guard3Nodes.add(p4GuardPredicate5);
      GuardedAction p4ga32 = new GuardedAction(new Guard(guard3Nodes), p4Action3);
      p4ga32.addClock(clockEvent);

      ArrayList<Assignment> action4Statements = new ArrayList<>();
      action4Statements.add(p4Statement6);
      action4Statements.add(p4Statement8);
      Action p4Action4 = new Action(action4Statements);
      List<Node> guard4Nodes = new ArrayList<>();
      guard4Nodes.add(Node.NOT(p4GuardPredicate1));
      guard4Nodes.add(Node.NOT(p4GuardPredicate2));
      guard4Nodes.add(p4GuardPredicate4);
      guard4Nodes.add(Node.NOT(p4GuardPredicate5));
      GuardedAction p4ga4 = new GuardedAction(new Guard(guard4Nodes), p4Action4);
      p4ga4.addClock(clockEvent);

      ArrayList<Assignment> action5Statements = new ArrayList<>();
      action5Statements.add(p4Statement4);
      action5Statements.add(p4Statement3);
      action5Statements.add(p4Statement6);
      Action p4Action5 = new Action(action5Statements);
      List<Node> guard5Nodes = new ArrayList<>();
      guard5Nodes.add(p4GuardPredicate2);
      guard5Nodes.add(Node.NOT(p4GuardPredicate1));
      guard5Nodes.add(Node.NOT(p4GuardPredicate4));
      GuardedAction p4ga5 = new GuardedAction(new Guard(guard5Nodes), p4Action5);
      p4ga5.addClock(clockEvent);

      Action p4Action6 = new Action(p4Statement7);
      GuardedAction p4ga6 = new GuardedAction(new Guard(guard5Nodes), p4Action6);
      p4ga6.addClock(clockEvent);

      List<Assignment> action7Statements = new ArrayList<>();
      action7Statements.add(p4Statement4);
      action7Statements.add(p4Statement3);
      action7Statements.add(p4Statement7);
      Action p4Action7 = new Action(action7Statements);
      List<Node> guard7Nodes = new ArrayList<>();
      guard7Nodes.add(Node.NOT(p4GuardPredicate1));
      guard7Nodes.add(p4GuardPredicate2);
      guard7Nodes.add(p4GuardPredicate4);
      GuardedAction p4ga7 = new GuardedAction(new Guard(guard7Nodes), p4Action7);
      p4ga7.addClock(clockEvent);

      Action p4Action9 = new Action(p4Statement7);
      List<Node> guard9Nodes = new ArrayList<>();
      guard9Nodes.add(Node.NOT(p4GuardPredicate1));
      guard9Nodes.add(p4GuardPredicate2);
      guard9Nodes.add(p4GuardPredicate4);
      guard9Nodes.add(p4GuardPredicate5);
      GuardedAction p4ga9 = new GuardedAction(new Guard(guard9Nodes), p4Action9);
      p4ga9.addClock(clockEvent);

      ArrayList<Assignment> action10Statements = new ArrayList<>();
      action10Statements.add(p4Statement6);
      action10Statements.add(p4Statement8);
      Action p4Action10 = new Action(action10Statements);
      List<Node> guard10Nodes = new ArrayList<>();
      guard10Nodes.add(Node.NOT(p4GuardPredicate1));
      guard10Nodes.add(p4GuardPredicate2);
      guard10Nodes.add(p4GuardPredicate4);
      guard10Nodes.add(Node.NOT(p4GuardPredicate5));
      GuardedAction p4ga10 = new GuardedAction(new Guard(guard10Nodes), p4Action10);
      p4ga10.addClock(clockEvent);

      ArrayList<Assignment> action11Statements = new ArrayList<>();
      action11Statements.add(p4Statement2);
      action11Statements.add(p4Statement3);
      Action p4Action11 = new Action(action11Statements);
      List<Node> guard11Nodes = new ArrayList<>();
      guard11Nodes.add(p4GuardPredicate1);
      guard11Nodes.add(Node.NOT(p4GuardPredicate2));
      guard11Nodes.add(Node.NOT(p4GuardPredicate4));
      GuardedAction p4ga11 = new GuardedAction(new Guard(guard11Nodes), p4Action11);
      p4ga11.addClock(clockEvent);

      ArrayList<Assignment> action12Statements = new ArrayList<>();
      action12Statements.add(p4Statement2);
      action12Statements.add(p4Statement3);
      action12Statements.add(p4Statement7);
      Action p4Action12 = new Action(action12Statements);
      List<Node> guard12Nodes = new ArrayList<>();
      guard12Nodes.add(p4GuardPredicate1);
      guard12Nodes.add(Node.NOT(p4GuardPredicate2));
      guard12Nodes.add(p4GuardPredicate4);
      GuardedAction p4ga121 = new GuardedAction(new Guard(guard12Nodes), p4Action12);
      p4ga121.addClock(clockEvent);

      guard12Nodes.add(p4GuardPredicate5);
      GuardedAction p4ga122 = new GuardedAction(new Guard(guard12Nodes), p4Action12);
      p4ga122.addClock(clockEvent);

      ArrayList<Assignment> action13Statements = new ArrayList<>();
      action13Statements.add(p4Statement8);
      action13Statements.add(p4Statement3);
      action13Statements.add(p4Statement6);
      Action p4Action13 = new Action(action13Statements);
      List<Node> guard13Nodes = new ArrayList<>();
      guard13Nodes.add(p4GuardPredicate1);
      guard13Nodes.add(Node.NOT(p4GuardPredicate2));
      guard13Nodes.add(p4GuardPredicate4);
      guard13Nodes.add(Node.NOT(p4GuardPredicate5));
      GuardedAction p4ga13 = new GuardedAction(new Guard(guard13Nodes), p4Action13);
      p4ga13.addClock(clockEvent);

      Action p4Action14 = new Action(action12Statements);
      List<Node> guard14Nodes = new ArrayList<>();
      guard14Nodes.add(p4GuardPredicate1);
      guard14Nodes.add(p4GuardPredicate2);
      guard14Nodes.add(Node.NOT(p4GuardPredicate4));
      GuardedAction p4ga14 = new GuardedAction(new Guard(guard14Nodes), p4Action14);
      p4ga14.addClock(clockEvent);

      Action p4Action15 = new Action(action12Statements);
      List<Node> guard15Nodes = new ArrayList<>();
      guard15Nodes.add(p4GuardPredicate1);
      guard15Nodes.add(p4GuardPredicate2);
      guard15Nodes.add(p4GuardPredicate4);
      guard15Nodes.add(p4GuardPredicate5);
      GuardedAction p4ga15 = new GuardedAction(new Guard(guard15Nodes), p4Action15);
      p4ga15.addClock(clockEvent);

      Action p4Action16 = new Action(action13Statements);
      List<Node> guard16Nodes = new ArrayList<>();
      guard16Nodes.add(p4GuardPredicate1);
      guard16Nodes.add(p4GuardPredicate2);
      guard16Nodes.add(p4GuardPredicate4);
      guard16Nodes.add(Node.NOT(p4GuardPredicate5));
      GuardedAction p4ga16 = new GuardedAction(new Guard(guard16Nodes), p4Action16);
      p4ga16.addClock(clockEvent);

      List<Assignment> action17Statements = new ArrayList<>();
      action17Statements.add(p4Statement2);
      action17Statements.add(p4Statement3);
      action17Statements.add(p4Statement6);
      action17Statements.add(p4Statement4);
      Action p4Action17 = new Action(action17Statements);
      ArrayList<Node> guard17Nodes = new ArrayList<>();
      guard17Nodes.add(p4GuardPredicate1);
      guard17Nodes.add(p4GuardPredicate2);
      guard17Nodes.add(Node.NOT(p4GuardPredicate4));
      GuardedAction p4ga17 = new GuardedAction(new Guard(guard17Nodes), p4Action17);
      p4ga17.addClock(clockEvent);

      List<Assignment> action18Statements = new ArrayList<>();
      action18Statements.add(p4Statement2);
      action18Statements.add(p4Statement3);
      action18Statements.add(p4Statement7);
      action18Statements.add(p4Statement4);
      Action p4Action18 = new Action(action18Statements);
      ArrayList<Node> guard18Nodes = new ArrayList<>();
      guard18Nodes.add(p4GuardPredicate1);
      guard18Nodes.add(p4GuardPredicate2);
      guard18Nodes.add(p4GuardPredicate4);
      guard18Nodes.add(p4GuardPredicate5);
      GuardedAction p4ga18 = new GuardedAction(new Guard(guard18Nodes), p4Action18);
      p4ga18.addClock(clockEvent);

      process4.createTransition(p4State1, p4State1, p4Reset);
      process4.createTransition(p4State2, p4State1, p4Reset);

      process4.createTransition(p4State1, p4State1, p4ga31);
      process4.createTransition(p4State2, p4State2, p4ga32);

      process4.createTransition(p4State2, p4State2, p4ga4);

      process4.createTransition(p4State1, p4State2, p4ga5);

      process4.createTransition(p4State2, p4State2, p4ga6);

      process4.createTransition(p4State1, p4State2, p4ga7);

      process4.createTransition(p4State2, p4State2, p4ga9);

      process4.createTransition(p4State2, p4State2, p4ga10);

      process4.createTransition(p4State1, p4State2, p4ga11);
      process4.createTransition(p4State2, p4State2, p4ga11);

      process4.createTransition(p4State1, p4State2, p4ga121);
      process4.createTransition(p4State2, p4State2, p4ga122);

      process4.createTransition(p4State2, p4State2, p4ga13);

      process4.createTransition(p4State2, p4State2, p4ga14);

      process4.createTransition(p4State2, p4State2, p4ga15);

      process4.createTransition(p4State2, p4State2, p4ga16);

      process4.createTransition(p4State1, p4State2, p4ga17);

      process4.createTransition(p4State1, p4State2, p4ga18);

      efsmModule.addNonInitialEfsm(process4);
    }

    {
      Efsm process5 = new Efsm(efsmModule, "B13_process5");
      process5.addMetaInfo(MetaInfoType.HDL_LANG, MetaInfoValue.LANG_VHDL);

      EfsmState startBitState =
          process5.createState(NodeUtils.getEquation(nextBit,
              new NodeValue(DataType.INTEGER.valueOf(String.valueOf(START_BIT), 16))));
      EfsmState stopBitState =
          process5.createState(NodeUtils.getEquation(nextBit,
              new NodeValue(DataType.INTEGER.valueOf(String.valueOf(STOP_BIT), 16))));
      EfsmState bit0State =
          process5.createState(NodeUtils.getEquation(nextBit,
              new NodeValue(DataType.INTEGER.valueOf(String.valueOf(BIT0), 16))));
      EfsmState bit1State =
          process5.createState(NodeUtils.getEquation(nextBit,
              new NodeValue(DataType.INTEGER.valueOf(String.valueOf(BIT1), 16))));
      EfsmState bit2State =
          process5.createState(NodeUtils.getEquation(nextBit,
              new NodeValue(DataType.INTEGER.valueOf(String.valueOf(BIT2), 16))));
      EfsmState bit3State =
          process5.createState(NodeUtils.getEquation(nextBit,
              new NodeValue(DataType.INTEGER.valueOf(String.valueOf(BIT3), 16))));
      EfsmState bit4State =
          process5.createState(NodeUtils.getEquation(nextBit,
              new NodeValue(DataType.INTEGER.valueOf(String.valueOf(BIT4), 16))));
      EfsmState bit5State =
          process5.createState(NodeUtils.getEquation(nextBit,
              new NodeValue(DataType.INTEGER.valueOf(String.valueOf(BIT5), 16))));
      EfsmState bit6State =
          process5.createState(NodeUtils.getEquation(nextBit,
              new NodeValue(DataType.INTEGER.valueOf(String.valueOf(BIT6), 16))));
      EfsmState bit7State =
          process5.createState(NodeUtils.getEquation(nextBit,
              new NodeValue(DataType.INTEGER.valueOf(String.valueOf(BIT7), 16))));

      process5.setInitialState(startBitState);

      // Guards
      Node p5GuardPredicate1 = ExampleEfsmUtils.constructEquality(sendEn, "1");
      Node p5GuardPredicate2 = new NodeOperation(StandardOperation.GREATER, txConta,
          ExampleEfsmUtils.constructValue(txConta, "104"));

      // Assignments
      List<Assignment> p5Statements1 =
          Arrays.asList(ExampleEfsmUtils.constructAssignment(txEnd, "0"),
              ExampleEfsmUtils.constructAssignment(dataOut, "0"),
              ExampleEfsmUtils.constructAssignment(txConta, "0"));

      List<Assignment> p5Statements2 =
          Arrays.asList(ExampleEfsmUtils.constructAssignment(txEnd, "0"),
              ExampleEfsmUtils.constructAssignment(dataOut, "1"));

      Assignment p5Statement21 = ExampleEfsmUtils.constructAssignment(txEnd, "0");

      List<Assignment> p5Statements3 = new ArrayList<>();
      p5Statements3.add(new Assignment(txConta, new NodeOperation(
          StandardOperation.ADD, txConta, ExampleEfsmUtils.constructValue(txConta, "1"))));

      List<Assignment> p5Statements4 =
          Collections.singletonList(ExampleEfsmUtils.constructAssignment(dataOut, "0"));

      List<Assignment> p5Statements5 = new ArrayList<>();
      p5Statements5.add(new Assignment(dataOut, new NodeOperation(
          StandardOperation.BVAND, outReg, new NodeOperation(StandardOperation.BVLSHL,
              ExampleEfsmUtils.constructValue(outReg, "1"), new NodeValue(DataType.INTEGER.valueOf(
                  "7", 16))))));

      List<Assignment> p5Statements6 = new ArrayList<>();
      p5Statements6.add(new Assignment(dataOut, new NodeOperation(
          StandardOperation.BVAND, outReg, new NodeOperation(StandardOperation.BVLSHL,
              ExampleEfsmUtils.constructValue(outReg, "1"), new NodeValue(DataType.INTEGER.valueOf(
                  "6", 16))))));

      ArrayList<Assignment> p5Statements7 = new ArrayList<>();
      p5Statements7.add(new Assignment(dataOut, new NodeOperation(
          StandardOperation.BVAND, outReg, new NodeOperation(StandardOperation.BVLSHL,
              ExampleEfsmUtils.constructValue(outReg, "1"), new NodeValue(DataType.INTEGER.valueOf(
                  "5", 16))))));

      List<Assignment> p5Statements8 = new ArrayList<>();
      p5Statements8.add(new Assignment(dataOut, new NodeOperation(
          StandardOperation.BVAND, outReg, new NodeOperation(StandardOperation.BVLSHL,
              ExampleEfsmUtils.constructValue(outReg, "1"), new NodeValue(DataType.INTEGER.valueOf(
                  "4", 16))))));

      List<Assignment> p5Statements9 = new ArrayList<>();
      p5Statements9.add(new Assignment(dataOut, new NodeOperation(
          StandardOperation.BVAND, outReg, new NodeOperation(StandardOperation.BVLSHL,
              ExampleEfsmUtils.constructValue(outReg, "1"), new NodeValue(DataType.INTEGER.valueOf(
                  "3", 16))))));

      List<Assignment> p5Statements10 = new ArrayList<>();
      p5Statements10.add(new Assignment(dataOut, new NodeOperation(
          StandardOperation.BVAND, outReg, new NodeOperation(StandardOperation.BVLSHL,
              ExampleEfsmUtils.constructValue(outReg, "1"), new NodeValue(DataType.INTEGER.valueOf(
                  "2", 16))))));

      List<Assignment> p5Statements11 = new ArrayList<>();
      p5Statements11.add(new Assignment(dataOut, new NodeOperation(
          StandardOperation.BVAND, outReg, new NodeOperation(StandardOperation.BVLSHL,
              ExampleEfsmUtils.constructValue(outReg, "1"), new NodeValue(DataType.INTEGER.valueOf(
                  "1", 16))))));

      List<Assignment> p5Statements12 = new ArrayList<>();
      p5Statements12.add(new Assignment(dataOut, new NodeOperation(
          StandardOperation.BVAND, outReg, new NodeOperation(StandardOperation.BVLSHL,
              ExampleEfsmUtils.constructValue(outReg, "1"), new NodeValue(DataType.INTEGER.valueOf(
                  "0", 16))))));

      List<Assignment> p5Statements13 =
          Arrays.asList(ExampleEfsmUtils.constructAssignment(dataOut, "1"),
              ExampleEfsmUtils.constructAssignment(txEnd, "1"));

      List<Assignment> p5Statements14 =
          Collections.singletonList(ExampleEfsmUtils.constructAssignment(txConta, "0"));

      // Guarded actions
      Action p5Action1 = new Action(p5Statements1);
      GuardedAction p5Reset = new GuardedAction(ExampleEfsmUtils.ALWAYS_TRUE_GUARD, p5Action1);
      p5Reset.addClock(resetEvent);

      List<Assignment> p5Action2Statements = new ArrayList<>();
      p5Action2Statements.addAll(p5Statements2);
      p5Action2Statements.addAll(p5Statements3);
      Action p5Action2 = new Action(p5Action2Statements);
      List<Node> p5ga2Nodes = new ArrayList<>();
      p5ga2Nodes.add(p5GuardPredicate1);
      p5ga2Nodes.add(Node.NOT(p5GuardPredicate2));
      GuardedAction p5ga2 = new GuardedAction(new Guard(p5ga2Nodes), p5Action2);
      p5ga2.addClock(clockEvent);

      Action p5Action3 = new Action(p5Statements2);
      GuardedAction p5ga3 = new GuardedAction(new Guard(Node.NOT(p5GuardPredicate1)), p5Action3);
      p5ga3.addClock(clockEvent);

      List<Node> p5CommonGuardNodes = new ArrayList<>();
      p5CommonGuardNodes.add(p5GuardPredicate1);
      p5CommonGuardNodes.add(p5GuardPredicate2);

      List<Assignment> p5Action4Statements = new ArrayList<>();
      p5Action4Statements.add(p5Statement21);
      p5Action4Statements.addAll(p5Statements4);
      p5Action4Statements.addAll(p5Statements14);
      Action p5Action4 = new Action(p5Action4Statements);
      GuardedAction p5ga4 = new GuardedAction(new Guard(p5CommonGuardNodes), p5Action4);
      p5ga4.addClock(clockEvent);

      List<Assignment> p5Action5Statements = new ArrayList<>();
      p5Action5Statements.add(p5Statement21);
      p5Action5Statements.addAll(p5Statements5);
      p5Action5Statements.addAll(p5Statements14);
      Action p5Action5 = new Action(p5Action5Statements);
      GuardedAction p5ga5 = new GuardedAction(new Guard(p5CommonGuardNodes), p5Action5);
      p5ga5.addClock(clockEvent);

      List<Assignment> p5Action6Statements = new ArrayList<>();
      p5Action6Statements.add(p5Statement21);
      p5Action6Statements.addAll(p5Statements6);
      p5Action6Statements.addAll(p5Statements14);
      Action p5Action6 = new Action(p5Action6Statements);
      GuardedAction p5ga6 = new GuardedAction(new Guard(p5CommonGuardNodes), p5Action6);
      p5ga6.addClock(clockEvent);

      List<Assignment> p5Action7Statements = new ArrayList<>();
      p5Action7Statements.add(p5Statement21);
      p5Action7Statements.addAll(p5Statements7);
      p5Action7Statements.addAll(p5Statements14);
      Action p5Action7 = new Action(p5Action7Statements);
      GuardedAction p5ga7 = new GuardedAction(new Guard(p5CommonGuardNodes), p5Action7);
      p5ga7.addClock(clockEvent);

      List<Assignment> p5Action8Statements = new ArrayList<>();
      p5Action8Statements.add(p5Statement21);
      p5Action8Statements.addAll(p5Statements8);
      p5Action8Statements.addAll(p5Statements14);
      Action p5Action8 = new Action(p5Action8Statements);
      GuardedAction p5ga8 = new GuardedAction(new Guard(p5CommonGuardNodes), p5Action8);
      p5ga8.addClock(clockEvent);

      List<Assignment> p5Action9Statements = new ArrayList<>();
      p5Action9Statements.add(p5Statement21);
      p5Action9Statements.addAll(p5Statements9);
      p5Action9Statements.addAll(p5Statements14);
      Action p5Action9 = new Action(p5Action9Statements);
      GuardedAction p5ga9 = new GuardedAction(new Guard(p5CommonGuardNodes), p5Action9);
      p5ga9.addClock(clockEvent);

      List<Assignment> p5Action10Statements = new ArrayList<>();
      p5Action10Statements.add(p5Statement21);
      p5Action10Statements.addAll(p5Statements10);
      p5Action10Statements.addAll(p5Statements14);
      Action p5Action10 = new Action(p5Action10Statements);
      GuardedAction p5ga10 = new GuardedAction(new Guard(p5CommonGuardNodes), p5Action10);
      p5ga10.addClock(clockEvent);

      List<Assignment> p5Action11Statements = new ArrayList<>();
      p5Action11Statements.add(p5Statement21);
      p5Action11Statements.addAll(p5Statements11);
      p5Action11Statements.addAll(p5Statements14);
      Action p5Action11 = new Action(p5Action11Statements);
      GuardedAction p5ga11 = new GuardedAction(new Guard(p5CommonGuardNodes), p5Action11);
      p5ga11.addClock(clockEvent);

      List<Assignment> p5Action12Statements = new ArrayList<>();
      p5Action12Statements.add(p5Statement21);
      p5Action12Statements.addAll(p5Statements12);
      p5Action12Statements.addAll(p5Statements14);
      Action p5Action12 = new Action(p5Action12Statements);
      GuardedAction p5ga12 = new GuardedAction(new Guard(p5CommonGuardNodes), p5Action12);
      p5ga12.addClock(clockEvent);

      List<Assignment> p5Action13Statements = new ArrayList<>();
      p5Action13Statements.addAll(p5Statements13);
      p5Action13Statements.addAll(p5Statements14);
      Action p5Action13 = new Action(p5Action13Statements);
      GuardedAction p5ga13 = new GuardedAction(new Guard(p5CommonGuardNodes), p5Action13);
      p5ga13.addClock(clockEvent);

      //Transitions
      process5.createTransition(startBitState, startBitState, p5Reset);
      process5.createTransition(bit0State, startBitState, p5Reset);
      process5.createTransition(bit1State, startBitState, p5Reset);
      process5.createTransition(bit2State, startBitState, p5Reset);
      process5.createTransition(bit3State, startBitState, p5Reset);
      process5.createTransition(bit4State, startBitState, p5Reset);
      process5.createTransition(bit5State, startBitState, p5Reset);
      process5.createTransition(bit6State, startBitState, p5Reset);
      process5.createTransition(bit7State, startBitState, p5Reset);
      process5.createTransition(stopBitState, startBitState, p5Reset);

      process5.createTransition(startBitState, startBitState, p5ga2);
      process5.createTransition(bit0State, bit0State, p5ga2);
      process5.createTransition(bit1State, bit1State, p5ga2);
      process5.createTransition(bit2State, bit2State, p5ga2);
      process5.createTransition(bit3State, bit3State, p5ga2);
      process5.createTransition(bit4State, bit4State, p5ga2);
      process5.createTransition(bit5State, bit5State, p5ga2);
      process5.createTransition(bit6State, bit6State, p5ga2);
      process5.createTransition(bit7State, bit7State, p5ga2);
      process5.createTransition(stopBitState, stopBitState, p5ga2);

      process5.createTransition(startBitState, startBitState, p5ga3);
      process5.createTransition(bit0State, bit0State, p5ga3);
      process5.createTransition(bit1State, bit1State, p5ga3);
      process5.createTransition(bit2State, bit2State, p5ga3);
      process5.createTransition(bit3State, bit3State, p5ga3);
      process5.createTransition(bit4State, bit4State, p5ga3);
      process5.createTransition(bit5State, bit5State, p5ga3);
      process5.createTransition(bit6State, bit6State, p5ga3);
      process5.createTransition(bit7State, bit7State, p5ga3);
      process5.createTransition(stopBitState, stopBitState, p5ga3);

      process5.createTransition(startBitState, bit0State, p5ga4);
      process5.createTransition(bit0State, bit1State, p5ga5);
      process5.createTransition(bit1State, bit2State, p5ga6);
      process5.createTransition(bit2State, bit3State, p5ga7);
      process5.createTransition(bit3State, bit4State, p5ga8);
      process5.createTransition(bit4State, bit5State, p5ga9);
      process5.createTransition(bit5State, bit6State, p5ga10);
      process5.createTransition(bit6State, bit7State, p5ga11);
      process5.createTransition(bit7State, stopBitState, p5ga12);
      process5.createTransition(stopBitState, startBitState, p5ga13);

      efsmModule.addNonInitialEfsm(process5);
    }

    B13.EFSM_MODEL.addModule(efsmModule);
  }

  private B13() {
  }
}
