/*
 * Copyright 2016 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ru.ispras.retrascope.util;

import org.junit.Assert;
import org.junit.Test;
import ru.ispras.fortress.data.DataType;
import ru.ispras.fortress.data.types.bitvector.BitVector;
import ru.ispras.fortress.expression.Node;
import ru.ispras.fortress.expression.NodeValue;

/**
 * Test for {@link NodeUtils} class.
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public class NodeUtilTestCase {

  @Test
  public void runTest() {

    runCastMethodTest();
  }

  private void runCastMethodTest() {

    final Node boolTrue = NodeValue.newBoolean(true);
    final Node intOne = NodeValue.newInteger(1);
    final Node intSeven = NodeValue.newInteger(7);
    final Node bvSeven = NodeValue.newBitVector(BitVector.valueOf("111"));
    final Node bvOne = NodeValue.newBitVector(BitVector.TRUE);

    /* integer -> boolean */
    Assert.assertTrue(
        isSatEquation(boolTrue, NodeUtils.castToDataType(intSeven, DataType.BOOLEAN)));

    /* bool -> integer */
    Assert.assertTrue(
        isSatEquation(intOne, NodeUtils.castToDataType(boolTrue, DataType.INTEGER)));

    /* bit vector -> boolean */
    Assert.assertTrue(
        isSatEquation(boolTrue, NodeUtils.castToDataType(bvSeven, DataType.BOOLEAN)));

    /* boolean -> bit vector */
    Assert.assertTrue(
        isSatEquation(bvOne, NodeUtils.castToDataType(boolTrue, DataType.BIT_VECTOR(1))));

    /* integer -> bit vector */
    Assert.assertTrue(
        isSatEquation(bvSeven, NodeUtils.castToDataType(intSeven, DataType.BIT_VECTOR(3))));

    /* bit vector -> integer */
    Assert.assertTrue(
        isSatEquation(intSeven, NodeUtils.castToDataType(bvSeven, DataType.INTEGER)));
  }

  private boolean isSatEquation(final Node ... exprs) {
    return SolverUtils.isSat(NodeUtils.getEquation(exprs));
  }
}
