`timescale 1ns/1ps

module fifo_testbench;
  reg         clk;
  reg         rst;
  reg         val_rd;
  reg         val_wr;
  reg  [31:0] data_in;
  wire        val_out;
  wire [31:0] data_out;
  wire        is_ready;
  wire        is_full;

  fifo target
  (
    .clk(clk),
    .rst(rst),
    .val_rd(val_rd),
    .val_wr(val_wr),
    .data_in(data_in),
    .val_out(val_out),
    .data_out(data_out),
    .is_ready(is_ready),
    .is_full(is_full)
  );

  initial rst = 0;
  initial clk = 0;
  always  clk = #10 ~clk;

  initial begin
    $dumpvars(4, fifo_testbench);
    $display("testing fifo...");

    val_rd  = 1'b0;
    val_wr  = 1'b0;
    data_in = 32'h0000_0000;

    #100;

    $display("reseting fifo...");
    rst = 1; #10; rst = 0;

    $display("is_ready=%b", is_ready);

    #100;

    $display("writing to fifo...");
    @(negedge clk);
    val_wr  <= 1;
    data_in <= 32'h1234_0000;
    @(posedge clk);
    @(negedge clk);
    val_wr  <= 0;
    data_in <= 32'h0000_0000;

    #100;

    $display("writing to fifo...");
    @(negedge clk);
    val_wr  <= 1;
    data_in <= 32'h1234_0001;
    @(posedge clk);
    @(negedge clk);
    val_wr  <= 0;
    data_in <= 32'h0000_0000;

    #100;

    $display("writing to fifo...");
    @(negedge clk);
    val_wr  <= 1;
    data_in <= 32'h1234_0002;
    @(posedge clk);
    @(negedge clk);
    val_wr  <= 0;
    data_in <= 32'h0000_0000;

    #100;

    $display("writing to fifo...");
    @(negedge clk);
    val_wr  <= 1;
    data_in <= 32'h1234_0003;
    @(posedge clk);
    @(negedge clk);
    val_wr  <= 0;
    data_in <= 32'h0000_0000;

    #100;

    @(negedge clk);
    val_rd  <= 1;
    @(posedge clk);
    @(negedge clk);
    val_rd  <= 0;
    
    wait (val_out);

    $display("reading from fifo %h...", data_out);

    #100;

    $display("writing to fifo...");
    @(negedge clk);
    val_wr  <= 1;
    data_in <= 32'h1234_0004;
    @(posedge clk);
    @(negedge clk);
    val_wr  <= 0;
    data_in <= 32'h0000_0000;

    #100;

    @(negedge clk);
    val_rd  <= 1;
    @(posedge clk);
    @(negedge clk);
    val_rd  <= 0;
    
    wait (val_out);

    $display("reading from fifo %h...", data_out);

    #100;

    $display("done");
    $finish;
  end
endmodule /* ram_testbench */

